'*****************************************************************
'  Visual Basic .NET Functions for IDAutomation Barcode Fonts
'   Copyright, 2003-2005 IDAutomation.com, Inc. All rights reserved.
'  Redistribution and use of this code in source and/or binary
'  forms, with or without modification, are permitted provided
'  that: (1) all copies of the source code retain the above
'  unmodified copyright notice and this entire unmodified
'  section of text, (2) You or Your organization owns a valid
'  Developer License to this product from IDAutomation.com
'  and, (3) when any portion of this code is bundled in any
'  form with an application, a valid notice must be provided
'  within the user documentation, start-up screen or in the
'  help-about section of the application that specifies
'  IDAutomation.com as the provider of the Software bundled
'  with the application.
'*****************************************************************

Imports System
Imports System.Drawing
Imports System.Drawing.Printing

Public Class IDAutomationEncoder
    'Constants for this class
    Private Const ASCII_START128_A As Integer = 203 'Start character for Code128 Char Set A
    Private Const ASCII_START128_B As Integer = 204 'Start character for Code128 Char Set B
    Private Const ASCII_START128_C As Integer = 205 'Start character for Code128 Char Set C
    Private Const ASCII_FNC1_128 As Integer = 202 'FNC1 character for Char Set C
    Private Const ASCII_STOP128 As Integer = 206 'Stop character for all Code128 character sets
    Private Const ASCII_SWITCH_TO_C As Integer = 199 'Character for switching to character set C
    Private Const ASCII_SWITCH_TO_A As Integer = 201 'Charcter for switchting to character set A
    Private Const ASCII_SWITCH_TO_B As Integer = 200 'Charcter for switchting to character set B
    Private Const ASCII_SPACE_CHAR As Integer = 194 'Space replacement character
    Private Const CODABAR_START_CHAR As Char = "A"c 'Start character for codabar
    Private Const CODABAR_STOP_CHAR As Char = "B"c 'Start character for codabar
    Private Const ASCII_START_I2OF5 As Integer = 203 'Start character for Interleaved 2 of 5
    Private Const ASCII_STOP_I2OF5 As Integer = 204 'Stop character for Interleaved 2 of 5
    Private Const DOUBLE_ZEROES As String = "00" 'Special characters that must be accounted for
    Private Const CHECK_DIGIT_MOD As Integer = 103 'This is what we divide by to get the check digit

    Private Enum enuEncodingType As Short
        EncodingA = 1
        EncodingB = 2
        EncodingC = 3
    End Enum

    'This is the font object that is passed between the print method and the print
    'event handler
    Private lclFont As Font
    'This is the text that will be printed from the printing event handler
    Private TextToPrint As String

    'This is a helper function that takes in a string value and determines if it is a
    'number.
    Private Function IsNumber(ByVal InNumber As String) As Boolean
        Dim idx As Integer = 0
        Dim strLen As Integer = InNumber.Length
        Dim InNumber2 As Char() = InNumber.ToCharArray()
        Dim RtrnValue As Boolean = True
        If strLen > 0 Then
            idx = 0
            'ORIGINAL LINE: for(idx=0;idx < strLen;idx += 1)
            'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
            Do While idx < strLen
                If Char.IsNumber(InNumber2(idx)) = False Then
                    RtrnValue = False
                End If
                idx += 1
            Loop
        Else
            RtrnValue = False
        End If
        Return RtrnValue
    End Function 'End IsNumber function

    ''' <summary>
    ''' calculates a mod 10 check digit from the input string
    ''' </summary>
    ''' <param name="input"></param>
    ''' <returns></returns>
    Private Function FindMod10Digit(ByVal input As String) As String
        Dim remainder As Integer
        Dim M10Factor As Integer = 0
        Dim M10WeightedTotal As Integer = 0
        Dim M10StringLength As Integer = 0
        Dim M10I As Integer = 0

        M10Factor = 3
        M10WeightedTotal = 0
        M10StringLength = input.Length

        For M10I = M10StringLength To 1 Step -1
            'Get the value of each number starting at the end
            'multiply by the weighting factor which is 3,1,3,1...
            'and add the sum together
            M10WeightedTotal = M10WeightedTotal + Convert.ToInt32(input.Substring(M10I - 1, 1)) * M10Factor
            'change factor for next calculation
            M10Factor = 4 - M10Factor
        Next M10I
        remainder = M10WeightedTotal Mod 10
        If remainder = 0 Then
            ' Zero is the smallest remainder of 10
            Return "0"
        Else
            'return (char) ((10 - remainder) + 48);  //the ASCII value follows sequentially after 48
            Return (10 - remainder).ToString()
        End If
    End Function 'end FindMod10Digit()

    ''' <summary>
    ''' Interprets tilde characters for special processing.  
    ''' </summary>
    ''' <param name="InputValue"></param>
    ''' <returns></returns>
    Private Function Code128TildeProcessing(ByVal InputValue As String) As String
        Dim ActualInputValue As String = "" 'this is what we are going to return
        'Loop thru looking for a tilde
        Dim LenDataToEncode As Integer = InputValue.Length
        Dim idx As Integer = 0
        Dim jdx As Integer = 0
        Dim rev As Integer = 0

        'ORIGINAL LINE: for(int idx = 0;idx < LenDataToEncode;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            If InputValue.Substring(idx, 1).Equals("~") = True AndAlso ((LenDataToEncode - idx) > 3) Then
                'We are either doing mod 10 processing or we are doing regular tilde processing to pull the ascii value
                If InputValue.Substring(idx + 1, 1).Equals("m") = True Then 'we are doing mod 10
                    Try
                        Dim tempValue As Integer = Convert.ToInt32(InputValue.Substring(idx + 2, 2))
                        If tempValue > 0 AndAlso tempValue <= idx Then 'make sure that they aren't going over the number of characters left in the string
                            'total number of characters before tilde.  We want to skip any non-numeric characters 
                            'when taking into account the number of characters to go back.  Meaning if they try to
                            'encode 1234A567~m06, we would calculate the mod 10 digit by using 234567.
                            Dim StringToCheck As String = InputValue.Substring(0, idx)
                            Dim StringToCalculate As String = ""
                            For jdx = StringToCheck.Length To 0 Step -1
                                If IsNumber(StringToCheck.Substring(jdx - 1, 1)) = True Then
                                    StringToCalculate &= StringToCheck.Substring(jdx - 1, 1)
                                    If StringToCalculate.Length = tempValue Then 'we've got all we need
                                        Exit For
                                    End If
                                End If
                            Next jdx

                            'We now have a string of data that we can calculate a mod 10 check digit with
                            If StringToCalculate.Length = tempValue Then
                                'StrintToCalculate is backwards.  flip it
                                Dim revString As System.Text.StringBuilder = New System.Text.StringBuilder()
                                For rev = StringToCalculate.Length To 1 Step -1
                                    revString.Append(StringToCalculate.Substring(rev - 1, 1))
                                Next rev

                                ActualInputValue &= FindMod10Digit(revString.ToString())
                                idx += 3
                            Else 'not the correct number of characters
                                ActualInputValue &= InputValue.Substring(idx, 1)
                            End If
                        Else
                            ActualInputValue &= InputValue.Substring(idx, 1)
                        End If
                    Catch
                        ActualInputValue &= InputValue.Substring(idx, 1)
                    End Try
                ElseIf InputValue.Substring(idx + 1, 1).Equals("~") = True Then
                    'The user really just wants to encode a tilde
                    ActualInputValue &= InputValue.Substring(idx, 1)
                    idx += 1
                Else 'we're looking at pulling an ascii value
                    Try
                        Dim tempValue As Integer = Convert.ToInt32(InputValue.Substring(idx + 1, 3))
                        If tempValue > 0 AndAlso tempValue <= 255 Then
                            ActualInputValue &= Microsoft.VisualBasic.Chr(tempValue)
                            idx += 3
                        Else
                            ActualInputValue &= InputValue.Substring(idx, 1)
                        End If
                    Catch
                        ActualInputValue &= InputValue.Substring(idx, 1)
                    End Try
                End If
            Else
                ActualInputValue &= InputValue.Substring(idx, 1)
            End If
            idx += 1
        Loop
        Return ActualInputValue
    End Function 'end Code128TildeProcessing()

    ''' add the start character, stop character, and check digit character and return the 
    ''' completed string 
    ''' </summary>
    ''' <param name="InputValue">The value to barcode</param>
    ''' <param name="InputValue">whether or not tildes are processed as special escape characters</param>
    ''' <returns></returns>
    Public Function Code128(ByVal InputValue As String, ByVal applyTilde As Boolean) As String
        ' There are 3 different character sets for Code128, A, B, & C. Code A allows for uppercase
        'characters, punctuation, numbers and several special functions such as a return or tab.
        'Code B encodes everything from ASCII 32 to ASCII 126. It allows for upper and lower case
        'letters, punctuation, numbers and a few select functions. Code C encodes only numbers and
        'the FNC1 function. Because the numbers are "interleaved" into pairs, two numbers are encoded
        'into every barcode character which makes it a very high density barcode. If the number to
        'encode is not an even number, a leading zero should be added.

        Dim EncodedData As String = "" 'This is the resulting data after processing the InputValue
        Dim LenDataToEncode As Integer = 0 'Length of InputValue
        Dim idx As Integer = 0 'for loop counter
        Dim AscCharacter As Integer = 0 'Ascii value of a particular character
        Dim ChrArray As Char() 'Character array used to determine ASCII values
        Dim OutStartCharacter As Char = " "c 'character representing the start character of the output string
        Dim OutStopCharacter As Char 'character representing the start character of the output string
        Dim OutCheckCharacter As Char 'character representing the check digit for the data
        Dim AsciiFnc1Found As Boolean 'Determines logic path in for loop if the FNC1 character is found
        Dim CurrentChars As String 'This is the character(s) that we are currently evaluating in the for loop
        Dim WeightedTotal As Integer = 0 'This is the value calculate to obtain the check digit
        Dim EncodingType As enuEncodingType = enuEncodingType.EncodingA 'type of character encodeing that we are doing
        Dim iCheckDigitValue As Integer = 0 'Value returned from WeightedTotal mod function

        Dim ActualInputValue As String = "" 'value of input string after tilde processing
        If applyTilde = True Then
            ActualInputValue = Code128TildeProcessing(InputValue)
        Else
            ActualInputValue = InputValue
        End If

        'Here we select character set A, B or C for the START character
        LenDataToEncode = ActualInputValue.Length

        'In order to get the ASCII value of the character, we move the first character
        'in the character array associated w/ the input string.  We then explicitly cast 
        'it to an int data type.  
        ChrArray = ActualInputValue.ToCharArray(0, 1)
        AscCharacter = Microsoft.VisualBasic.Asc(ChrArray(0))

        'Determine the Code set of 128
        If AscCharacter < 32 Then 'Automatically has to be Code A
            OutStartCharacter = Microsoft.VisualBasic.Chr(ASCII_START128_A)
            EncodingType = enuEncodingType.EncodingA
        End If

        If (AscCharacter > 31 AndAlso AscCharacter < 127) OrElse AscCharacter = 197 Then
            OutStartCharacter = Microsoft.VisualBasic.Chr(ASCII_START128_B)
            EncodingType = enuEncodingType.EncodingB
        End If

        'Check and see if the first 4 digits are numbers.  Since Code C is pairs of interleaved numbers
        If LenDataToEncode > 4 AndAlso IsNumber(ActualInputValue.Substring(0, 4)) = True Then
            OutStartCharacter = Microsoft.VisualBasic.Chr(ASCII_START128_C)
            EncodingType = enuEncodingType.EncodingC
        End If

        If AscCharacter = ASCII_FNC1_128 OrElse (AscCharacter > 211 AndAlso AscCharacter < 218) Then '202 is the FNC1, with this Start C is mandatory
            OutStartCharacter = Microsoft.VisualBasic.Chr(ASCII_START128_C)
            EncodingType = enuEncodingType.EncodingC
        End If

        'Set the stop character of the string 
        OutStopCharacter = Microsoft.VisualBasic.Chr(ASCII_STOP128)

        'loop through all characters and determine if we need to switch character sets or encode special functions
        For idx = 0 To LenDataToEncode - 1

            'check for FNC1 in any set
            AsciiFnc1Found = False

            'Get the Ascii value of the current letter

            ChrArray = ActualInputValue.ToCharArray(idx, 1)
            AscCharacter = Microsoft.VisualBasic.Asc(ChrArray(0))
            If AscCharacter = ASCII_FNC1_128 OrElse (AscCharacter > 211 AndAlso AscCharacter < 218) Then
                ' Format and print various data:
                'If we get here we bypass the next set of if statements.

                AsciiFnc1Found = True
                'Add the FNC1 character to the output string 
                EncodedData = EncodedData + (Microsoft.VisualBasic.Chr(ASCII_FNC1_128))
            End If
            'Checking for switch to character set C.  In order to determine if we are switching to 
            'set C, we are checking if we have more than 2 characters left to check, & the digit is 
            'a number & the next character is a number and the ongoing 4 characters constitue a number 
            'OR
            'we are checking that this is not the last item, and the current char is a number, and 
            'the next character is a number and we are already encodint with Set C.
            'AND
            'AsciiFnc1Found = false  -- 
            If ((idx <= (LenDataToEncode - 4)) AndAlso (IsNumber(ActualInputValue.Substring(idx, 1)) = True) AndAlso (IsNumber(ActualInputValue.Substring(idx + 1, 1)) = True) AndAlso (IsNumber(ActualInputValue.Substring(idx, 4)) = True)) OrElse ((idx < LenDataToEncode - 1) AndAlso (IsNumber(ActualInputValue.Substring(idx, 1)) = True) AndAlso (IsNumber(ActualInputValue.Substring(idx + 1, 1)) = True) AndAlso (EncodingType = enuEncodingType.EncodingC)) AndAlso (AsciiFnc1Found = False) Then
                'check for switching to character set C
                'switch to set C if not already in it
                If EncodingType <> enuEncodingType.EncodingC Then
                    'Add the switch to set C character 
                    EncodedData = EncodedData + (Microsoft.VisualBasic.Chr(ASCII_SWITCH_TO_C))
                End If 'End check for type c

                EncodingType = enuEncodingType.EncodingC

                'We need to check for double zeros
                CurrentChars = ActualInputValue.Substring(idx, 2)
                If CurrentChars = DOUBLE_ZEROES Then
                    AscCharacter = ASCII_SPACE_CHAR
                Else
                    AscCharacter = Convert.ToInt32(CurrentChars)

                End If 'end else check for 00

                If AscCharacter < 95 AndAlso AscCharacter > 0 Then
                    AscCharacter = AscCharacter + 32
                    EncodedData = EncodedData + Microsoft.VisualBasic.Chr(AscCharacter)
                Else
                    If AscCharacter > 94 Then
                        If AscCharacter <> ASCII_SPACE_CHAR Then
                            AscCharacter = AscCharacter + 100
                        End If
                        EncodedData = EncodedData + Microsoft.VisualBasic.Chr(AscCharacter)
                    Else
                        If AscCharacter = 0 Then
                            EncodedData = EncodedData + Microsoft.VisualBasic.Chr(ASCII_SPACE_CHAR)
                        End If
                    End If 'end else checking for space character
                End If 'end else checking for between 0 & 95

                idx = idx + 1
            Else 'endif for character C check
                'Checking for char set A
                If ((idx <= LenDataToEncode) AndAlso (AscCharacter < 31)) OrElse ((EncodingType = enuEncodingType.EncodingA) AndAlso (AscCharacter > 32) AndAlso (AscCharacter < 96)) Then
                    If EncodingType <> enuEncodingType.EncodingA Then
                        EncodedData = EncodedData + Microsoft.VisualBasic.Chr(ASCII_SWITCH_TO_A)
                    End If
                    EncodingType = enuEncodingType.EncodingA

                    'Get the ASCII value of the next character
                    ChrArray = ActualInputValue.ToCharArray(idx, 1)
                    AscCharacter = Microsoft.VisualBasic.Asc(ChrArray(0))

                    'set the CurrentValue to the number of String CurrentChar
                    If AscCharacter = 32 Then
                        EncodedData = EncodedData + Microsoft.VisualBasic.Chr(ASCII_SPACE_CHAR)
                    Else
                        If AscCharacter < 32 Then
                            AscCharacter = AscCharacter + 96
                            EncodedData = EncodedData + Microsoft.VisualBasic.Chr(AscCharacter)
                        ElseIf AscCharacter > 32 Then
                            EncodedData = EncodedData + Microsoft.VisualBasic.Chr(AscCharacter)
                        End If
                    End If
                Else
                    If (idx <= LenDataToEncode) AndAlso ((AscCharacter > 31 AndAlso AscCharacter < 127) OrElse AscCharacter = 197) Then
                        If EncodingType <> enuEncodingType.EncodingB Then
                            'Write switch character for Set B
                            EncodedData = EncodedData + Microsoft.VisualBasic.Chr(ASCII_SWITCH_TO_B)
                        End If

                        EncodingType = enuEncodingType.EncodingB

                        'Get the ASCII value of the next character
                        ChrArray = ActualInputValue.ToCharArray(idx, 1)
                        AscCharacter = Microsoft.VisualBasic.Asc(ChrArray(0))

                        If AscCharacter = 32 Then
                            EncodedData = EncodedData + Microsoft.VisualBasic.Chr(ASCII_SPACE_CHAR)
                        Else
                            EncodedData = EncodedData + Microsoft.VisualBasic.Chr(AscCharacter)
                        End If
                    End If 'End switch to B character set
                End If 'end else from checking for character set A & B
            End If
        Next 'end loop thru characters

        'Now that we have all of the characters encoded & the start & stop digits, we need to calculate
        'the check digit.

        'Calculate Modulo 103 Check Digit
        'Set WeightedTotal to the value of the start character
        'WeightedTotal = (CInt(OutStartCharacter)) - 100
        WeightedTotal = Convert.ToInt32(OutStartCharacter) - 100

        'Change the LenDataToEncode to the data that we will include in the  output.
        LenDataToEncode = EncodedData.Length

        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx<LenDataToEncode;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            'Get the ASCII value of each character
            ChrArray = EncodedData.ToCharArray(idx, 1)
            AscCharacter = Microsoft.VisualBasic.Asc(ChrArray(0))

            'Get the Code 128 value of CurrentChar according to chart
            If AscCharacter < 135 Then
                AscCharacter = AscCharacter - 32
            Else
                If AscCharacter = 194 Then
                    AscCharacter = 0
                End If
                If AscCharacter > 134 Then
                    AscCharacter = AscCharacter - 100
                End If
            End If
            'multiply by the weighting character
            AscCharacter = AscCharacter * (idx + 1)

            WeightedTotal = WeightedTotal + AscCharacter

            idx += 1
        Loop 'End for loop to get Weighted Total

        'divide the WeightedTotal by 103 and get the remainder, this is the CheckDigitValue
        iCheckDigitValue = WeightedTotal Mod CHECK_DIGIT_MOD

        'Now that we have the CheckDigitValue, find the corresponding ASCII character from the table
        If iCheckDigitValue < 95 AndAlso iCheckDigitValue > 0 Then
            iCheckDigitValue = iCheckDigitValue + 32
        Else 'else check digit not between 0 & 95
            If iCheckDigitValue > 94 Then
                iCheckDigitValue = iCheckDigitValue + 100
            Else
                If iCheckDigitValue = 0 Then
                    iCheckDigitValue = ASCII_SPACE_CHAR
                End If
            End If 'end if check digit > 94
        End If 'end if check digit between 0 & 95
        OutCheckCharacter = Microsoft.VisualBasic.Chr(iCheckDigitValue)

        EncodedData = OutStartCharacter + EncodedData + OutCheckCharacter + OutStopCharacter
        'MessageBox.Show( EncodedData );

        Return EncodedData
    End Function 'end Code128()

    ''' <summary>
    ''' This method will convert data encoded for Code128 and return the human readable form
    ''' of the code 
    ''' </summary>
    ''' <param name="InputValue">The value that is to be barcoded</param>
    ''' <param name="applyTilde">Determines if tilde characters should be interpreted as escape sequence characters</param>
    ''' <returns></returns>
    Public Function Code128HR(ByVal InputValue As String, ByVal applyTilde As String) As String
        Dim sbReturnString As System.Text.StringBuilder = New System.Text.StringBuilder()

        Dim ActualInputValue As String = "" 'value of input string after tilde processing
        If applyTilde = True Then
            ActualInputValue = Code128TildeProcessing(InputValue)
        Else
            ActualInputValue = InputValue
        End If

        Dim charDataToEncode As Char() = ActualInputValue.ToCharArray()
        Dim FNCDone As Boolean
        Dim i As Integer = 0
        'ORIGINAL LINE: for(int i = 0;i < charDataToEncode.Length;i += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While i < charDataToEncode.Length
            FNCDone = False

            'need to make sure there are enough characters left in ActualInputValue and add AI characters
            If i <= ActualInputValue.Length - 3 AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) = 212) Then
                sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 2) & ") ")
                i += 2
            ElseIf i <= ActualInputValue.Length - 4 AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) = 213) Then
                sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 3) & ") ")
                i += 3
            ElseIf i <= ActualInputValue.Length - 5 AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) = 214) Then
                sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 4) & ") ")
                i += 4
            ElseIf i <= ActualInputValue.Length - 6 AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) = 215) Then
                sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 5) & ") ")
                i += 5
            ElseIf i <= ActualInputValue.Length - 7 AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) = 216) Then
                sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 6) & ") ")
                i += 6
            ElseIf i <= ActualInputValue.Length - 8 AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) = 217) Then
                sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 7) & ") ")
                i += 7
            ElseIf i <= ActualInputValue.Length - 4 AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) = 202) AndAlso IsNumber(ActualInputValue.Substring(i + 1, 1)) = True AndAlso IsNumber(ActualInputValue.Substring(i + 2, 1)) = True Then
                Dim Next2CharacterValue As Integer = 0
                Try
                    Next2CharacterValue = Convert.ToInt32(ActualInputValue.Substring(i + 1, 2))

                    If Next2CharacterValue = 80 OrElse Next2CharacterValue = 81 OrElse (Next2CharacterValue <= 34 AndAlso Next2CharacterValue >= 31) Then
                        sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 4) & ") ")
                        i += 4
                        FNCDone = True
                    End If
                    If FNCDone = False Then
                        If (Next2CharacterValue <= 49 AndAlso Next2CharacterValue >= 40) OrElse (Next2CharacterValue <= 25 AndAlso Next2CharacterValue >= 23) Then
                            sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 3) & ") ")
                            i += 3
                            FNCDone = True
                        End If
                    End If
                    If FNCDone = False Then
                        If (Next2CharacterValue <= 30 AndAlso Next2CharacterValue >= 0) OrElse (Next2CharacterValue <= 99 AndAlso Next2CharacterValue >= 90) Then
                            sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 2) & ") ")
                            i += 2
                            FNCDone = True
                        End If
                    End If
                    'If no AI was detected, set default to 4 digit AI:
                    If FNCDone = False Then
                        sbReturnString.Append(" (" & ActualInputValue.Substring(i + 1, 4) & ") ")
                        i += 4
                        FNCDone = True
                    End If
                Catch
                    'took an error converting the 2 digits.  Just add the current character then
                    sbReturnString.Append(ActualInputValue.Substring(i, 1))
                End Try
            ElseIf (Microsoft.VisualBasic.Asc(charDataToEncode(i)) >= 32) AndAlso (Microsoft.VisualBasic.Asc(charDataToEncode(i)) <= 126) Then
                sbReturnString.Append(ActualInputValue.Substring(i, 1))
            ElseIf Microsoft.VisualBasic.Asc(charDataToEncode(i)) < 32 Then
                sbReturnString.Append(" ")
            End If
            i += 1
        Loop 'end for loop
        Return sbReturnString.ToString()
    End Function
    'This method will create Code128 Set A encoded data from the Input string.  Code128A allows for uppercase 
    'characters, punctuation, numbers and several special functions such as a return or tab. It may be 
    'necessary to use set A to encode these functions in a barcode
    Public Function Code128A(ByVal InputValue As String) As String
        Dim Code128AStartChar As Char 'start character for Code 128A
        Dim Code128AStopChar As Char 'stop character for Code 128A
        Dim Code128ACheckDigit As Char 'check digit for Code 128A
        Dim EncodedData As String = "" 'This is the encoded data that we wwill return
        Dim idx As Integer = 0 'for loop counter, to loop thru characters passed in
        Dim WeightedTotal As Integer = 0 'used to get weighted total of characters for check digit
        Dim LenDataToEncode As Integer = 0 'Len of string passed in
        Dim InputData As Char() 'Character array of input string passed in, so that we can evaluate each charactr
        Dim iCurrentChar As Integer 'Ascii value of current character
        ' Here we select character set A. We can simply convert the ASCII value to a character
        '* by casting it as a char type

        Code128AStartChar = Microsoft.VisualBasic.Chr(ASCII_START128_A)
        Code128AStopChar = Microsoft.VisualBasic.Chr(ASCII_STOP128)

        ' <<<< Calculate Modulo 103 Check Digit >>>> 
        ' Set WeightedTotal to the value of the start character 
        WeightedTotal = ASCII_START128_A - 100

        LenDataToEncode = InputValue.Length
        InputData = InputValue.ToCharArray()

        idx = 0
        'ORIGINAL LINE: for(idx=0; idx < LenDataToEncode; idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            ' Get the ASCII value of each character 
            iCurrentChar = Microsoft.VisualBasic.Asc(InputData(idx))

            'Let's calculate the weighted total.
            If iCurrentChar < 135 Then
                iCurrentChar = iCurrentChar - 32
            ElseIf iCurrentChar > 134 Then
                iCurrentChar = iCurrentChar - 100
            End If

            'Multiply by the weighting value and add it to the total
            WeightedTotal = WeightedTotal + (iCurrentChar * (idx + 1))
            idx += 1
        Loop

        ' divide the WeightedTotal by 103 and get the remainder,
        '* this is the CheckDigitValue

        iCurrentChar = WeightedTotal Mod 103

        ' Now that we have the CheckDigitValue, find the corresponding ASCII character
        'from the table
        If iCurrentChar < 95 AndAlso iCurrentChar > 0 Then
            iCurrentChar = iCurrentChar + 32
        ElseIf iCurrentChar > 94 Then
            iCurrentChar = iCurrentChar + 100
        ElseIf iCurrentChar = 0 Then
            iCurrentChar = ASCII_SPACE_CHAR
        End If

        'Convert the ASCII value to the character representation.
        Code128ACheckDigit = Microsoft.VisualBasic.Chr(iCurrentChar)


        ' Check for spaces and print ASCII 194 instead 
        ' place changes in EncodedData so that it can be returned 
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenDataToEncode;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            If InputData(idx) = " "c Then
                'Check for space
                EncodedData = EncodedData & Microsoft.VisualBasic.Chr(ASCII_SPACE_CHAR)
            Else
                EncodedData = EncodedData & InputData(idx)
            End If
            idx += 1
        Loop
        EncodedData = Code128AStartChar & EncodedData & Code128ACheckDigit + Code128AStopChar
        ' Return the EncodedData 
        Return EncodedData
    End Function 'End Code128A

    'This method will create Code128 Set B encoded data from the Input string.  Code128 B is the most common because 
    'it encodes everything from ASCII 32 to ASCII 126. It allows for upper and lower case letters, punctuation, 
    'numbers and a few select functions.
    Public Function Code128B(ByVal InputValue As String) As String
        Dim Code128BStartChar As Char 'start character for Code 128A
        Dim Code128BStopChar As Char 'stop character for Code 128A
        Dim Code128BCheckDigit As Char 'check digit for Code 128A
        Dim EncodedData As String = "" 'This is the encoded data that we wwill return
        Dim idx As Integer = 0 'for loop counter, to loop thru characters passed in
        Dim WeightedTotal As Integer = 0 'used to get weighted total of characters for check digit
        Dim LenDataToEncode As Integer = 0 'Len of string passed in
        Dim InputData As Char() 'Character array of input string passed in, so that we can evaluate each charactr
        Dim iCurrentChar As Integer 'Ascii value of current character
        ' Here we select character set B. We can simply convert the ASCII value to a character
        '* by casting it as a char type

        Code128BStartChar = Microsoft.VisualBasic.Chr(ASCII_START128_B)
        Code128BStopChar = Microsoft.VisualBasic.Chr(ASCII_STOP128)

        ' <<<< Calculate Modulo 103 Check Digit >>>> 
        ' Set WeightedTotal to the value of the start character 
        WeightedTotal = ASCII_START128_B - 100

        LenDataToEncode = InputValue.Length
        InputData = InputValue.ToCharArray()

        idx = 0
        'ORIGINAL LINE: for(idx=0; idx < LenDataToEncode; idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            ' Get the ASCII value of each character 
            iCurrentChar = Microsoft.VisualBasic.Asc(InputData(idx))

            'Let's calculate the weighted total.
            If iCurrentChar < 135 Then
                iCurrentChar = iCurrentChar - 32
            ElseIf iCurrentChar > 134 Then
                iCurrentChar = iCurrentChar - 100
            End If

            'Multiply by the weighting value and add it to the total
            WeightedTotal = WeightedTotal + (iCurrentChar * (idx + 1))
            idx += 1
        Loop

        ' divide the WeightedTotal by 103 and get the remainder,
        '* this is the CheckDigitValue

        iCurrentChar = WeightedTotal Mod 103

        ' Now that we have the CheckDigitValue, find the corresponding ASCII character
        'from the table
        If iCurrentChar < 95 AndAlso iCurrentChar > 0 Then
            iCurrentChar = iCurrentChar + 32
        ElseIf iCurrentChar > 94 Then
            iCurrentChar = iCurrentChar + 100
        ElseIf iCurrentChar = 0 Then
            iCurrentChar = ASCII_SPACE_CHAR
        End If

        'Convert the ASCII value to the character representation.
        Code128BCheckDigit = Microsoft.VisualBasic.Chr(iCurrentChar)


        ' Check for spaces and print ASCII 194 instead 
        ' place changes in EncodedData so that it can be returned 
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenDataToEncode;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            If InputData(idx) = " "c Then
                'Check for space
                EncodedData = EncodedData & Microsoft.VisualBasic.Chr(ASCII_SPACE_CHAR)
            Else
                EncodedData = EncodedData & InputData(idx)
            End If

            idx += 1
        Loop

        ' Return the EncodedData 
        EncodedData = Code128BStartChar & EncodedData & Code128BCheckDigit + Code128BStopChar
        Return EncodedData

    End Function 'End Code128B

    'This routine will create a Code128 Set C encoded data string.  Set C encodes only numbers and the 
    'FNC1 function. Because the numbers are "interleaved" into pairs, two numbers are encoded into every 
    'barcode character which makes it a very high density barcode. If the number to encode is not an even 
    'number, a leading zero should be added. 
    Public Function Code128C(ByVal InputValue As String) As String
        Dim Code128CStartChar As Char 'start character for Code 128A
        Dim Code128CStopChar As Char 'stop character for Code 128A
        Dim Code128CCheckDigit As Char 'check digit for Code 128A
        Dim EncodedData As String = "" 'This is the encoded data that we wwill return
        Dim idx As Integer = 0 'for loop counter, to loop thru characters passed in
        Dim WeightedTotal As Integer = 0 'used to get weighted total of characters for check digit
        Dim LenDataToEncode As Integer = 0 'Len of string passed in
        Dim InputData As Char() 'Character array of input string passed in, so that we can evaluate each charactr
        Dim GoodData As String = "" 'This is the good data from the InputValue, once we strip out dashes
        Dim iCurrentChars As Integer 'The integer value of the 2 characters we are currently working with
        Dim sCurrentChars As String 'The sting representation of the 2 characters we are working with.
        Dim WeightFactor As Integer = 0 'used to calculate the check digit
        ' Here we select character set C. We can simply convert the ASCII value to a character
        '* by casting it as a char type

        Code128CStartChar = Microsoft.VisualBasic.Chr(ASCII_START128_C)
        Code128CStopChar = Microsoft.VisualBasic.Chr(ASCII_STOP128)

        ' <<<< Calculate Modulo 103 Check Digit >>>> 
        ' Set WeightedTotal to the value of the start character 
        WeightedTotal = ASCII_START128_C - 100

        LenDataToEncode = InputValue.Length
        InputData = InputValue.ToCharArray()

        'First check to make sure data is number and remove and dashes
        idx = 0
        'ORIGINAL LINE: for(idx=0; idx < LenDataToEncode; idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            If (Char.IsNumber(InputData(idx))) = False Then
                'make sure it is not a dash.  If it is a dash, remove it, otherwise get out of the function
                'If InputData(idx) <> "-"c Then
                '    EncodedData = "Non Numeric Data Passed In"
                '    Return EncodedData
                'End If 'Otherwise it is a dash and we can just move on
            Else
                GoodData = GoodData & InputData(idx)
            End If
            idx += 1
        Loop

        'Ensure that there is an even number of digits.  If not we need to add a zero to the end
        If (GoodData.Length Mod 2) <> 0 Then
            GoodData = "0" & GoodData
        End If

        'Calculate Mod 103 Weighted total check digit value, using only the Good Data
        LenDataToEncode = GoodData.Length
        WeightFactor = 1
        idx = 0
        'ORIGINAL LINE: for(idx=0; idx < LenDataToEncode; idx = idx + 2)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenDataToEncode
            ' Get the value of the current character plus the next 
            sCurrentChars = GoodData.Substring(idx, 2)
            iCurrentChars = Convert.ToInt32(sCurrentChars)

            'Multiply by the weighting value and add it to the total
            WeightedTotal = WeightedTotal + (iCurrentChars * WeightFactor)
            WeightFactor = WeightFactor + 1

            'Let's calculate the weighted total.
            If iCurrentChars < 95 AndAlso iCurrentChars > 0 Then
                iCurrentChars = iCurrentChars + 32
            ElseIf iCurrentChars > 94 Then
                iCurrentChars = iCurrentChars + 100
            ElseIf iCurrentChars = 0 Then
                iCurrentChars = ASCII_SPACE_CHAR
            End If

            'Set the return value
            EncodedData = EncodedData & Microsoft.VisualBasic.Chr(iCurrentChars)
            idx = idx + 2
        Loop

        ' divide the WeightedTotal by 103 and get the remainder, this is the CheckDigitValue
        iCurrentChars = WeightedTotal Mod 103

        ' Now that we have the CheckDigitValue, find the corresponding ASCII character
        'from the table
        If iCurrentChars < 95 AndAlso iCurrentChars > 0 Then
            iCurrentChars = iCurrentChars + 32
        ElseIf iCurrentChars > 94 Then
            iCurrentChars = iCurrentChars + 100
        ElseIf iCurrentChars = 0 Then
            iCurrentChars = ASCII_SPACE_CHAR
        End If

        'Convert the ASCII value to the character representation.
        Code128CCheckDigit = Microsoft.VisualBasic.Chr(iCurrentChars)

        ' Return the EncodedData 
        EncodedData = Code128CStartChar & EncodedData & Code128CCheckDigit + Code128CStopChar
        Return EncodedData
    End Function 'End Code 128C

    'The Codabar bar code symbology is used for various numeric bar coding applications 
    'including libraries, blood banks and parcels. Codabar was designed for character 
    'self-checking eliminating the requirement for checksum characters.  However, 
    'checksum characters in the Codabar barcode are optional and they do maximize data 
    'integrity. 
    'The symbology of the Codabar character set consists of bar code symbols representing 
    'characters 0-9, Letters A to D and the following symbols: -  .  $  /  +.  Additional
    'data can be encoded in the actual choice of start and stop codes. The letters 
    'A, B C and D are used for start and stop codes. For example, if you want to print the
    'data 2727 in the Codabar barcode font you would print A2727B with the Codabar font selected.																																																																																				
    Public Function Codabar(ByVal InputValue As String) As String
        Dim GoodData As String = "" 'valid characters pulled from InputValue
        Dim idx As Integer = 0 'for loop coutner
        Dim LenOfData As Integer = 0 'Length of strings
        Dim CurrentChar As String = "" 'Current character we are evaluating

        LenOfData = InputValue.Length
        'Loop through input and remove any invalid characters.  
        ' Check to make sure data is numeric, $, +, -, /, or :, and remove all others. 
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            CurrentChar = InputValue.Substring(idx, 1)
            If CurrentChar = "$" OrElse CurrentChar = "+" OrElse CurrentChar = "-" OrElse CurrentChar = "/" OrElse CurrentChar = ":" OrElse IsNumber(CurrentChar) = True Then
                GoodData = GoodData & CurrentChar
            End If
            idx += 1
        Loop

        Return CODABAR_START_CHAR & GoodData & CODABAR_STOP_CHAR
    End Function 'End Codabar function

    'Interleaved 2 of 5 is a numeric only barcode used for encoding pairs of numbers in a high density 
    'barcode format. Interleaved 2 of 5 barcodes always contain an even number of digits because a 
    'single Interleaved 2 of 5 barcode character represents two numbers to achieve the high density.
    'If a number containing an odd number of digits has to be encoded, a leading zero must be added 
    'to produce an even number of digits in the Interleaved 2 of 5 bar code.
    Public Function Interleaved2of5(ByVal InputValue As String) As String
        Dim GoodData As String = "" 'valid numeric Data from Input Value
        Dim LenOfData As Integer = 0 'Length of the string we are processing
        Dim idx As Integer = 0 'for loop counter
        Dim CurrentChar As String = "" 'Character(s) that we are evaluating
        Dim OutData As String = "" 'This is the interleaved data which we will output
        Dim iCurrentChar As Integer = 0 'The integer value of the data we are interleaving

        LenOfData = InputValue.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            CurrentChar = InputValue.Substring(idx, 1)
            If IsNumber(CurrentChar) = True Then
                GoodData = GoodData & CurrentChar
            End If
            idx += 1
        Loop

        'If the length of our data is an odd number of characters, we must prepend a 0
        If (GoodData.Length Mod 2) <> 0 Then
            GoodData = "0" & GoodData
        End If

        LenOfData = GoodData.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx = idx + 2)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            'We need to get the actual value of the 2 characters (numbers) that we would like to
            'interleave
            CurrentChar = GoodData.Substring(idx, 2)
            iCurrentChar = Convert.ToInt32(CurrentChar)

            'Populate the string representing the encoded data
            If iCurrentChar < 94 Then
                OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 33))
            Else
                OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 103))
            End If
            idx = idx + 2
        Loop
        OutData = (Microsoft.VisualBasic.Chr(ASCII_START_I2OF5) & OutData & Microsoft.VisualBasic.Chr(ASCII_STOP_I2OF5))
        Return OutData
    End Function 'end Interleaved2of5()

    'This is similar to the standard Interleaved 2 of 5 method, but requires a MOD 10 check 
    'digit as part of the bar code.
    Public Function Interleaved2of5Mod10(ByVal InputValue As String) As String
        Dim GoodData As String = "" 'valid numeric Data from Input Value
        Dim LenOfData As Integer = 0 'Length of the string we are processing
        Dim idx As Integer = 0 'for loop counter
        Dim CurrentChar As String = "" 'Character(s) that we are evaluating
        Dim OutData As String = "" 'This is the interleaved data which we will output
        Dim iCurrentChar As Integer = 0 'The integer value of the data we are interleaving
        Dim WeightedTotal As Integer = 0 'Weighted total used for check digit
        Dim WeightFactor As Integer = 0 'Factor used to create weighted total
        Dim CheckDigit As Integer = 0 'This is the Check sum digit

        LenOfData = InputValue.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            CurrentChar = InputValue.Substring(idx, 1)
            If IsNumber(CurrentChar) = True Then
                GoodData = GoodData & CurrentChar
            End If

            idx += 1
        Loop
        'We need to come up with an even number of digits(including the check digit) so what 
        'if the value passed in has an even number of digits we have to add a zero to make it odd.
        'Once we have an odd number of digits we will add the check character to make it even again.

        If (GoodData.Length Mod 2) = 0 Then
            GoodData = "0" & GoodData
        End If

        'Calculate the Check Sum character
        LenOfData = GoodData.Length
        WeightFactor = 3
        'We have to start ata the end and work backwards to generate the check digit
        For idx = (LenOfData - 1) To 0 Step -1
            'Get the current character
            CurrentChar = GoodData.Substring(idx, 1)
            iCurrentChar = Convert.ToInt32(CurrentChar)

            WeightedTotal = WeightedTotal + (iCurrentChar * WeightFactor)

            WeightFactor = 4 - WeightFactor

        Next idx

        'This will give us the mod 10 
        CheckDigit = WeightedTotal Mod 10
        If CheckDigit <> 0 Then
            CheckDigit = (10 - CheckDigit)
        Else
            CheckDigit = 0
        End If

        'Add the ascii character representation of the check digit to the GoodData string
        GoodData = GoodData & CheckDigit.ToString()
        LenOfData = GoodData.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx = idx + 2)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            'We need to get the actual value of the 2 characters (numbers) that we would like to
            'interleave
            CurrentChar = GoodData.Substring(idx, 2)
            iCurrentChar = Convert.ToInt32(CurrentChar)

            'Populate the string representing the encoded data
            If iCurrentChar < 94 Then
                OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 33))
            Else
                OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 103))
            End If
            idx = idx + 2
        Loop
        OutData = (Microsoft.VisualBasic.Chr(ASCII_START_I2OF5) & OutData & Microsoft.VisualBasic.Chr(ASCII_STOP_I2OF5))
        Return OutData
    End Function 'end Interleaved2of5Mod10


    'Code 39 is commonly used for various barcoding labels such as name badges, inventory 
    'and industrial applications. The Code 39 barcode is the easiest to use of alpha-numeric barcodes
    'and is designed for character self-checking, eliminating the requirement for check character 
    'calculations. 
    'The symbology of the Code 39 character set consists of bar code symbols representing 
    'characters 0-9, A-Z, the space character and the following symbols:  - , . , $ , / , + , %. 
    'In addition, the full 128 character ASCII character set can be encoded in code 39.
    Public Function Code39(ByVal InputValue As String) As String
        Dim LenOfData As Integer = 0 'Length of the string we are processing
        Dim OutData As String = "" 'This is the interleaved data which we will output

        LenOfData = InputValue.Length
        'Replace any spaces with = signs
        InputValue = InputValue.Replace(" ", "=")

        OutData = "!" & InputValue & "!"

        Return OutData
    End Function 'End Code 39

    'Code39mod43 is the same as the standard Code39, but it includes a check digit based on a mod 43 calculation
    Public Function Code39mod43(ByVal InputValue As String) As String
        Dim LenOfData As Integer = 0 'Length of the string we are processing
        Dim idx As Integer = 0 'for loop counter
        Dim OutData As String = "" 'This is the interleaved data which we will output
        Dim iCurrentChar As Integer = 0 'The integer value of the data
        Dim cInputValue As Char() 'character array of our input data
        Dim WeightedTotal As Integer = 0 'Weighted Total to generate check digit
        Dim CheckDigit As Integer = 0 'This is the check digit for the data

        LenOfData = InputValue.Length
        'Convert the string to upper case
        InputValue = InputValue.ToUpper()
        cInputValue = InputValue.ToCharArray()

        'We need to loop through all characters and ensure that we are only using valid characters
        'from the input string
        idx = 0
        'ORIGINAL LINE: for(idx=0;idx<LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            'Get the ASCII value of the current character
            iCurrentChar = Microsoft.VisualBasic.Asc(cInputValue(idx))

            If (iCurrentChar < 58) AndAlso (iCurrentChar > 47) Then ' 0-9 
                OutData = OutData & cInputValue(idx)
            ElseIf (iCurrentChar < 91) AndAlso (iCurrentChar > 64) Then  ' A-Z 
                OutData = OutData & cInputValue(idx)
            ElseIf iCurrentChar = 32 Then  ' Space 
                OutData = OutData & cInputValue(idx)
            ElseIf iCurrentChar = 45 Then  ' - 
                OutData = OutData & cInputValue(idx)
            ElseIf iCurrentChar = 46 Then  '. 
                OutData = OutData & cInputValue(idx)
            ElseIf iCurrentChar = 36 Then  ' $ 
                OutData = OutData & cInputValue(idx)
            ElseIf iCurrentChar = 47 Then  ' / 
                OutData = OutData & cInputValue(idx)
            ElseIf iCurrentChar = 43 Then  ' + 
                OutData = OutData & cInputValue(idx)
            ElseIf iCurrentChar = 37 Then  ' % 
                OutData = OutData & cInputValue(idx)
            End If
            idx += 1
        Loop

        'Now get the weighted total of the characters to create the check digit
        WeightedTotal = 0
        LenOfData = OutData.Length
        cInputValue = OutData.ToCharArray()
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            'Get the ASCII value of the current character
            iCurrentChar = Microsoft.VisualBasic.Asc(cInputValue(idx))

            ' Get the value of CurrentChar according to MOD43 standards
            If (iCurrentChar < 58) AndAlso (iCurrentChar > 47) Then ' 0-9 
                iCurrentChar = iCurrentChar - 48
            ElseIf (iCurrentChar < 91) AndAlso (iCurrentChar > 64) Then  ' A-Z 
                iCurrentChar = iCurrentChar - 55
            ElseIf iCurrentChar = 32 Then  ' Space 
                iCurrentChar = 38
            ElseIf iCurrentChar = 45 Then ' - 
                iCurrentChar = 36
            ElseIf iCurrentChar = 46 Then  '. 
                iCurrentChar = 37
            ElseIf iCurrentChar = 36 Then  ' $ 
                iCurrentChar = 39
            ElseIf iCurrentChar = 47 Then  ' / 
                iCurrentChar = 40
            ElseIf iCurrentChar = 43 Then  ' + 
                iCurrentChar = 41
            ElseIf iCurrentChar = 37 Then  ' % 
                iCurrentChar = 42
            ElseIf iCurrentChar = 32 Then  ' To print the barcode symbol representing a space you will to type or print "=" (the equal character) instead of a space character.
                iCurrentChar = 61
            End If

            WeightedTotal = WeightedTotal + iCurrentChar
            idx += 1
        Loop
        CheckDigit = WeightedTotal Mod 43

        'Replace any space characters with an equal sign
        OutData = OutData.Replace(" ", "=")

        ' Assign values to the check digit 
        If CheckDigit < 10 Then ' 0-9 
            CheckDigit = CheckDigit + 48
        ElseIf (CheckDigit < 36) AndAlso (CheckDigit > 9) Then  ' A-Z 
            CheckDigit = CheckDigit + 55
        ElseIf CheckDigit = 38 Then ' Space 
            CheckDigit = 61
        ElseIf CheckDigit = 36 Then ' - 
            CheckDigit = 45
        ElseIf CheckDigit = 37 Then '. 
            CheckDigit = 46
        ElseIf CheckDigit = 39 Then ' $ 
            CheckDigit = 36
        ElseIf CheckDigit = 40 Then ' / 
            CheckDigit = 47
        ElseIf CheckDigit = 41 Then ' + 
            CheckDigit = 43
        ElseIf CheckDigit = 42 Then ' % 
            CheckDigit = 37
        End If

        OutData = "!" & OutData & (Microsoft.VisualBasic.Chr(CheckDigit)) & "!"
        Return OutData
    End Function
    'The POSTNet barcode can bo one of 3 different combinations.
    'The 5 digit POSTNET bar code consists of the start/stop character, 5 digit ZIP code data, check digit 
    'and the start/stop character. The  ZIP+4 POSTNET bar code consists of the start/stop character, 
    '9 digit ZIP code data, check digit and the start/stop character. The  DPBC POSTNET bar code consists 
    'of the start/stop character, 9 digit ZIP code data, two DPBC numbers, check digit and the start/stop 
    'character.
    'The start/stop character must be used at the beginning and ending of every POSTNET barcode. 
    'For best performance, the parentheses are used for start and stop characters; "(" for start and  ")" 
    'for stop. Start and stop codes are combined for the "!, *, s, or S" characters. 
    'The last digit of the printed POSTNET barcode symbol is a check digit. The check digit is obtained by 
    'determining the number that when added to the sum of all numbers of the data in the POSTNET code will 
    'produce a multiple of 10.  
    Public Function POSTNet(ByVal InputValue As String) As String
        Dim OutData As String = "" 'this is the return value of the function
        Dim GoodData As String = "" 'This is the InputValue stripped of invalid characters
        Dim LenOfData As Integer = 0 'Length of a string
        Dim idx As Integer = 0 'for loop counter
        Dim WeightedTotal As Integer = 0 'Weighted Total for check digit
        Dim iCurrentChar As Integer = 0 'Integer value of current number in InputValue
        Dim CheckDigit As Integer = 0 'Ascii value of check digit

        ' Check to make sure data is numeric and remove dashes, etc. 
        LenOfData = InputValue.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Add all numbers to GoodData string 
            If (IsNumber(InputValue.Substring(idx, 1))) = True Then
                GoodData = GoodData & InputValue.Substring(idx, 1)
            End If
            idx += 1
        Loop

        ' <<<< Calculate Check Digit >>>> 
        WeightedTotal = 0
        LenOfData = GoodData.Length

        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Get the value of each number 
            iCurrentChar = Convert.ToInt32(GoodData.Substring(idx, 1))

            ' add the values together 
            WeightedTotal = WeightedTotal + iCurrentChar
            idx += 1
        Loop

        ' Find the CheckDigit by finding the number + weightedTotal that = a multiple of 10 
        ' divide by 10, get the remainder and subtract from 10 
        CheckDigit = WeightedTotal Mod 10
        If CheckDigit <> 0 Then
            CheckDigit = (10 - CheckDigit)
        Else
            CheckDigit = 0
        End If

        OutData = "(" & GoodData & (CheckDigit.ToString()) & ")"

        ' Return OutData 
        Return OutData
    End Function 'End POSTNet

    'This is a numeric only bar code used to create European Article Numbering System (EAN) barcodes, which 
    'are a superset of U.P.C. EAN-8 consists of 8 digits for small packages. 
    Public Function EAN8(ByVal InputValue As String) As String
        Dim LenOfData As Integer = 0 'len of string
        Dim idx As Integer = 0 'for loop counter
        Dim OutData As String = "" 'this is the return value of the function
        Dim GoodData As String = "" 'This is the InputValue stripped of invalid characters
        Dim WeightedTotal As Integer = 0 'Weighted Total for check digit
        Dim iCurrentChar As Integer = 0 'Integer value of current number in InputValue
        Dim CheckDigit As Integer = 0 'Ascii value of check digit
        Dim WeightFactor As Integer = 0 'Weighted multiplier for check digit
        Dim cInputValue As Char() 'Character array of a string

        ' Check to make sure data is numeric and remove dashes, etc. 
        LenOfData = InputValue.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Add all numbers to GoodData string 
            If (IsNumber(InputValue.Substring(idx, 1))) = True Then
                GoodData = GoodData & InputValue.Substring(idx, 1)
            End If
            idx += 1
        Loop

        If GoodData.Length <> 7 Then
            GoodData = "Cannot process; you MUST enter a 7 digit NUMBER for this type of barcode. Do not use any spaces or dashes."
            Return GoodData
        End If

        ' <<<< Calculate Check Digit >>>> 
        WeightFactor = 3
        WeightedTotal = 0
        LenOfData = GoodData.Length
        cInputValue = GoodData.ToCharArray()

        For idx = LenOfData - 1 To 0 Step -1
            ' Get the value of each number 
            iCurrentChar = Microsoft.VisualBasic.Val(cInputValue(idx))

            ' multiply by the weighting factor which is 3,1,3,1... and add the sum together 
            WeightedTotal = WeightedTotal + (iCurrentChar * WeightFactor)
            WeightFactor = 4 - WeightFactor
        Next idx

        ' Find the CheckDigit by finding the number + weightedTotal that = a multiple of 10 
        ' divide by 10, get the remainder and subtract from 10 
        CheckDigit = WeightedTotal Mod 10
        If CheckDigit <> 0 Then
            CheckDigit = (10 - CheckDigit)
        Else
            CheckDigit = 0
        End If

        'Add the check digit to our data
        GoodData = GoodData & CheckDigit.ToString()

        ' Now that have the total number including the check digit, determine character to print for proper barcoding 
        LenOfData = GoodData.Length

        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Get the ASCII value of each number 
            cInputValue = GoodData.ToCharArray()
            iCurrentChar = Microsoft.VisualBasic.Asc(cInputValue(idx))
            Select Case idx + 1
                Case 1 ' For the first character print the normal guard pattern and then the barcode without the human readable character 
                    OutData = OutData & "(" & (Microsoft.VisualBasic.Chr(iCurrentChar))
                Case 2
                    OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar))
                Case 3
                    OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar))
                Case 4 ' Print the center guard pattern after the 6th character 
                    OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar)) & "*"
                Case 5
                    OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 27))
                Case 6
                    OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 27))
                Case 7
                    OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 27))
                Case 8 ' Print the check digit as 8th character + normal guard pattern 
                    OutData = OutData & (Microsoft.VisualBasic.Chr(iCurrentChar + 27)) & "("
                Case Else
                    Exit Select
            End Select 'End Switch
            idx += 1
        Loop 'End for loop
        Return OutData
    End Function 'end EAN8

    Public Function EAN13(ByVal InputValue As String) As String
        Dim OutData As String = "" 'this is the return value of the function
        Dim GoodData As String = "" 'This is the InputValue stripped of invalid characters
        Dim LenOfData As Integer = 0 'Length of a string
        Dim idx As Integer = 0 'for loop counter
        Dim WeightedTotal As Integer = 0 'Weighted Total for check digit
        Dim WeightFactor As Integer = 0 'The multiplier to determine the check digit.
        Dim iCurrentChar As Integer = 0 'Integer value of current number in InputValue
        Dim CheckDigit As Integer = 0 'Ascii value of check digit
        Dim EAN5AddOn As String = "" 'Add on characters for longer EAN symbols
        Dim EAN2AddOn As String = "" 'Add on characters for longer EAN symbols
        Dim Encoding As String = "" 'Encoding format
        Dim LeadingDigit As Integer = 0 'Ascii value of leading digit
        Dim cInputValue As Char() 'Character array of a string
        Dim CurrentEncoding As String = "" 'encoding in looop
        Dim temp As String = "" 'Temp string
        Dim EANAddOnToPrint As String = "" 'Extended EAN information
        Dim EANaddchar As Char 'Character to add to extended EAN information
        Dim iEANaddchar As Integer = 0 'integer value of above character
        Dim tempArray As Char() 'array to hold character of strings

        ' Check to make sure data is numeric and remove dashes, etc. 
        LenOfData = InputValue.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Add all numbers to GoodData string 
            If (IsNumber(InputValue.Substring(idx, 1))) = True Then
                GoodData = GoodData & InputValue.Substring(idx, 1)
            End If
            idx += 1
        Loop

        ' DataToEncode = OnlyCorrectData 
        ' Remove check digits if they added one 
        'The value passed in will only be 12, 13, 15, or 18 digits.  The 
        LenOfData = GoodData.Length
        If LenOfData = 13 Then 'we have the 12 digit EAN plus a check digit
            GoodData = GoodData.Substring(0, 12)
        ElseIf LenOfData = 15 Then  'we have the 12 digit EAN plus a check digit plus a 2 digit add on
            GoodData = GoodData.Substring(0, 12) & GoodData.Substring(13, 2)
        ElseIf LenOfData = 18 Then  'we have the 12 digit EAN plus a check digit plus a 5 digit add on
            GoodData = GoodData.Substring(0, 12) & GoodData.Substring(13, 5)
        ElseIf GoodData.Length < 12 Or GoodData.Length > 18 Then
            GoodData = "000000500000"
        End If
        
        'Now based on the new length of Good Data we can determine what the Add on value is.
        ' End sub if incorrect number 
        If GoodData.Length = 17 Then
            'Check digit has already been eliminated so this is just the EAN code and the 5 digit add on
            EAN5AddOn = GoodData.Substring(12, 5)
        ElseIf GoodData.Length = 14 Then
            'Check digit has already been eliminated so this is just the EAN code and the 2 digit add on
            EAN2AddOn = GoodData.Substring(12, 2)
        End If

        'Now set GoodData to only the EAN code (the first 12 characters
        GoodData = GoodData.Substring(0, 12)

        ' ' 
        WeightFactor = 3
        WeightedTotal = 0

        ' <<<< Calculate Check Digit >>>> 
        ' Get the value of each number starting at the end 
        LenOfData = GoodData.Length
        cInputValue = GoodData.ToCharArray()

        For idx = (LenOfData - 1) To 0 Step -1
            ' Get the ascii value of each number starting at the end 
            iCurrentChar = Convert.ToInt32(GoodData.Substring(idx, 1))
            ' multiply by the weighting factor which is 3,1,3,1... 
            ' and add the sum together 
            WeightedTotal = WeightedTotal + (iCurrentChar * WeightFactor)
            ' change factor for next calculation 
            WeightFactor = 4 - WeightFactor
        Next idx

        ' Find the CheckDigitValue by finding the number + weightedTotal that = a multiple of 10 
        ' divide by 10, get the remainder and subtract from 10 
        CheckDigit = WeightedTotal Mod 10
        If CheckDigit <> 0 Then
            CheckDigit = (10 - CheckDigit)
        Else
            CheckDigit = 0
        End If

        ' Now we must encode the leading digit into the left half of the EAN-13 symbol 
        ' by using variable parity between character sets A and B 
        cInputValue = GoodData.ToCharArray()

        'LeadingDigit = ((int) cInputValue[0] ) - 48 ;
        LeadingDigit = Convert.ToInt32(GoodData.Substring(0, 1))

        Select Case LeadingDigit
            Case 0
                Encoding = "AAAAAACCCCCC"
            Case 1
                Encoding = "AABABBCCCCCC"
            Case 2
                Encoding = "AABBABCCCCCC"
            Case 3
                Encoding = "AABBBACCCCCC"
            Case 4
                Encoding = "ABAABBCCCCCC"
            Case 5
                Encoding = "ABBAABCCCCCC"
            Case 6
                Encoding = "ABBBAACCCCCC"
            Case 7
                Encoding = "ABABABCCCCCC"
            Case 8
                Encoding = "ABABBACCCCCC"
            Case 9
                Encoding = "ABBABACCCCCC"
                Exit Select
        End Select 'end switch addresseing encoding type

        ' add the check digit to the end of the barcode & remove the leading digit 
        GoodData = GoodData.Substring(1, 11) + (CheckDigit.ToString())

        ' Now that we have the total number including the check digit, determine character to print for proper barcoding: 
        LenOfData = GoodData.Length
        cInputValue = GoodData.ToCharArray()

        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Get the ASCII value of each number excluding the first number because it is encoded with variable parity 
            iCurrentChar = Microsoft.VisualBasic.Asc(cInputValue(idx))
            CurrentEncoding = Encoding.Substring(idx, 1)

            ' Print different barcodes according to the location of the CurrentChar and CurrentEncoding 
            Select Case CurrentEncoding
                Case "A"
                    OutData = OutData + cInputValue(idx)
                Case "B"
                    iCurrentChar = iCurrentChar + 17
                    OutData = OutData + Microsoft.VisualBasic.Chr(iCurrentChar)

                Case "C"
                    iCurrentChar = iCurrentChar + 27
                    OutData = OutData + Microsoft.VisualBasic.Chr(iCurrentChar)
                Case Else
                    Exit Select
            End Select 'end switch to checking encoding type
            ' add in the 1st character along with guard patterns 
            Select Case idx
                Case 0
                    ' For the LeadingDigit print the human readable character, the normal guard pattern and then the rest of the barcode 
                    'This is the character we will put before OutData
                    If LeadingDigit > 4 Then
                        temp = LeadingDigit.ToString()
                        tempArray = temp.ToCharArray()
                        iCurrentChar = Microsoft.VisualBasic.Asc(tempArray(0))
                        iCurrentChar = iCurrentChar + 64
                        OutData = (Microsoft.VisualBasic.Chr(iCurrentChar)) & "(" & OutData
                    ElseIf LeadingDigit < 5 Then
                        temp = LeadingDigit.ToString()
                        tempArray = temp.ToCharArray()
                        iCurrentChar = Microsoft.VisualBasic.Asc(tempArray(0))
                        iCurrentChar = iCurrentChar + 37
                        OutData = (Microsoft.VisualBasic.Chr(iCurrentChar)) & "(" & OutData
                    End If
                Case 5 ' Print the center guard pattern after the 6th character 
                    OutData = OutData & "*"
                Case 11 ' For the last character (12) print the the normal guard pattern after the barcode 
                    OutData = OutData & "("
                Case Else
                    Exit Select
            End Select 'end switch for determining main output for EAN code
            idx += 1
        Loop 'end for loop that encodes the data

        If EAN5AddOn.Length = 5 Then
            ' Get check digit for add on 
            WeightFactor = 3
            WeightedTotal = 0
            For idx = (EAN5AddOn.Length - 1) To 0 Step -1
                ' Get the value of each number starting at the end 
                iCurrentChar = Convert.ToInt32(EAN5AddOn.Substring(idx, 1))

                ' multiply by the weighting factor which is 3,9,3,9. 
                ' and add the sum together 
                If WeightFactor = 3 Then
                    WeightedTotal = WeightedTotal + (iCurrentChar * 3)
                End If
                If WeightFactor = 1 Then
                    WeightedTotal = WeightedTotal + (iCurrentChar * 9)
                End If
                ' change factor for next calculation 
                WeightFactor = 4 - WeightFactor
            Next idx 'end check digit for loop for EAN 5 character add on

            ' Find the CheckDigit by extracting the right-most number from weightedTotal 
            temp = WeightedTotal.ToString()
            CheckDigit = Convert.ToInt32(temp.Substring(temp.Length - 1, 1))

            ' Now we must encode the add-on CheckDigit into the number sets 
            ' by using variable parity between character sets A and B 
            Select Case CheckDigit
                Case 0
                    Encoding = "BBAAA"
                Case 1
                    Encoding = "BABAA"
                Case 2
                    Encoding = "BAABA"
                Case 3
                    Encoding = "BAAAB"
                Case 4
                    Encoding = "ABBAA"
                Case 5
                    Encoding = "AABBA"
                Case 6
                    Encoding = "AAABB"
                Case 7
                    Encoding = "ABABA"
                Case 8
                    Encoding = "ABAAB"
                Case 9
                    Encoding = "AABAB"
                    Exit Select
            End Select 'end switch for EAN 5 encoding type

            ' Now that we have the total number including the check digit, determine character to print for proper barcoding: 
            LenOfData = EAN5AddOn.Length
            EANAddOnToPrint = ""
            For idx = 0 To LenOfData - 1
                ' Get the value of each number it is encoded with variable parity 
                iCurrentChar = Convert.ToInt32(EAN5AddOn.Substring(idx, 1))
                CurrentEncoding = Encoding.Substring(idx, 1)

                If CurrentEncoding = "A" Then ' Print different barcodes according to the location of the CurrentChar and CurrentEncoding 
                    Select Case iCurrentChar
                        Case 0
                            iEANaddchar = 34
                        Case 1
                            iEANaddchar = 35
                        Case 2
                            iEANaddchar = 36
                        Case 3
                            iEANaddchar = 37
                        Case 4
                            iEANaddchar = 38
                        Case 5
                            iEANaddchar = 44
                        Case 6
                            iEANaddchar = 46
                        Case 7
                            iEANaddchar = 47
                        Case 8
                            iEANaddchar = 58
                        Case 9
                            iEANaddchar = 59
                            Exit Select

                    End Select 'end character switch for EAN 5 character for encoding A
                    EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                    EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                ElseIf CurrentEncoding = "B" Then
                    Select Case iCurrentChar
                        Case 0
                            iEANaddchar = 122
                        Case 1
                            iEANaddchar = 61
                        Case 2
                            iEANaddchar = 63
                        Case 3
                            iEANaddchar = 64
                        Case 4
                            iEANaddchar = 91
                        Case 5
                            iEANaddchar = 92
                        Case 6
                            iEANaddchar = 93
                        Case 7
                            iEANaddchar = 95
                        Case 8
                            iEANaddchar = 123
                        Case 9
                            iEANaddchar = 125
                            Exit Select

                    End Select 'End switch for EAN5 add on w/ B encoding
                    EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                    EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                    Select Case idx ' add in the space & add-on guard pattern 
                        Case 0
                            iEANaddchar = 43
                            EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                            EANAddOnToPrint = EANaddchar + EANAddOnToPrint
                            iEANaddchar = 32
                            EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                            EANAddOnToPrint = EANaddchar + EANAddOnToPrint
                            iEANaddchar = 33
                            EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                            EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                        Case 1
                            iEANaddchar = 33
                            EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                            EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                        Case 2
                            iEANaddchar = 33
                            EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                            EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                        Case 3
                            iEANaddchar = 33
                            EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                            EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                        Case 4
                            'do nothing
                            Exit Select
                    End Select 'end add character switch
                End If 'end else for Encoding B for EAN add on 5 characters
            Next 'end loop thru 5 digit supplement
        End If  'end if ean5 add on

        'Check for 2 digit add on
        If EAN2AddOn.Length = 2 Then
            'Get the actual value of the EAN2 add on
            iEANaddchar = Convert.ToInt32(EAN2AddOn)

            ' Get encoding for add on 
            For idx = 0 To 99 Step 4
                If iEANaddchar = idx Then
                    Encoding = "AA"
                ElseIf iEANaddchar = (idx + 1) Then
                    Encoding = "AB"
                ElseIf iEANaddchar = (idx + 2) Then
                    Encoding = "BA"
                ElseIf iEANaddchar = (idx + 3) Then
                    Encoding = "BB"
                End If
            Next idx 'end for loop determining encoding type

            LenOfData = EAN2AddOn.Length ' Now that we have the total number including the encoding determine what to print 
            idx = 0
            'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
            'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
            Do While idx < LenOfData
                ' Get the value of each number it is encoded with variable parity 
                iCurrentChar = Convert.ToInt32(EAN2AddOn.Substring(idx, 1))
                CurrentEncoding = Encoding.Substring(idx, 1)

                ' Print different barcodes according to the location of the CurrentChar and CurrentEncoding 
                If CurrentEncoding = "A" Then
                    Select Case iCurrentChar
                        Case 0
                            iEANaddchar = 34
                        Case 1
                            iEANaddchar = 35
                        Case 2
                            iEANaddchar = 36
                        Case 3
                            iEANaddchar = 37
                        Case 4
                            iEANaddchar = 38
                        Case 5
                            iEANaddchar = 44
                        Case 6
                            iEANaddchar = 46
                        Case 7
                            iEANaddchar = 47
                        Case 8
                            iEANaddchar = 58
                        Case 9
                            iEANaddchar = 59
                            Exit Select
                    End Select 'end switch encoding A
                    EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                    EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                ElseIf CurrentEncoding = "B" Then
                    Select Case iCurrentChar
                        Case 0
                            iEANaddchar = 122
                        Case 1
                            iEANaddchar = 61
                        Case 2
                            iEANaddchar = 63
                        Case 3
                            iEANaddchar = 64
                        Case 4
                            iEANaddchar = 91
                        Case 5
                            iEANaddchar = 92
                        Case 6
                            iEANaddchar = 93
                        Case 7
                            iEANaddchar = 95
                        Case 8
                            iEANaddchar = 123
                        Case 9
                            iEANaddchar = 125
                            Exit Select
                    End Select 'end switch encoding B
                    EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                    EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                End If 'end else if for encoding B
                'add in the space & add-on guard pattern 
                If idx = 0 Then
                    iEANaddchar = 43
                    EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                    EANAddOnToPrint = EANaddchar + EANAddOnToPrint

                    iEANaddchar = 33
                    EANaddchar = Microsoft.VisualBasic.Chr(iEANaddchar)
                    EANAddOnToPrint = EANAddOnToPrint + EANaddchar
                End If 'end if idx ==1
                idx = idx + 1
            Loop 'end loop thru addon
        End If 'end if for check for 2 digit add on

        'Get OutData String
        OutData = OutData + EANAddOnToPrint

        Return OutData
    End Function 'end ean13()

    'The symbology of the MSI Plessey character set consists of bar code symbols representing the numbers 
    '0-9, the start character and the stop character. In the MSI Plessey font, the parentheses are used 
    'for start and stop characters.  A check digit character is included with this encoding before the 
    'closing parentheses.
    Public Function MSI(ByVal InputValue As String) As String
        Dim OutData As String = "" 'this is the return value of the function
        Dim GoodData As String = "" 'This is the InputValue stripped of invalid characters
        Dim LenOfData As Integer = 0 'Length of a string
        Dim idx As Integer = 0 'for loop counter
        Dim WeightedTotal As Integer = 0 'Weighted Total for check digit
        Dim CheckDigit As Integer = 0 'Ascii value of check digit
        Dim OddNumbers As String = "" 'string of odd numbers from input value
        Dim iOddNumbers As Integer = 0 'integr of the OddNumbers string
        Dim EvenNumbers As Integer = 0 'sum of all even numbers in input value

        ' Check to make sure data is numeric and remove dashes, etc. 
        LenOfData = InputValue.Length

        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Add all numbers to OnlyCorrectData string 
            If (IsNumber(InputValue.Substring(idx, 1))) = True Then
                GoodData = GoodData & InputValue.Substring(idx, 1)
            End If
            idx += 1
        Loop

        ' <<<< Calculate Check Digit >>>> 
        WeightedTotal = 0
        LenOfData = GoodData.Length

        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Get the value of each EVEN number, 1st number is even & add them together 
            If ((idx + 1) Mod 2) = 1 Then 'means this is odd
                OddNumbers = OddNumbers & GoodData.Substring(idx, 1)
            ElseIf ((idx + 1) Mod 2) = 0 Then  'means it's even
                EvenNumbers = EvenNumbers + Convert.ToInt32(GoodData.Substring(idx, 1))
            End If
            idx += 1
        Loop

        ' Multiply odd numbers gathered by 2 
        iOddNumbers = (Convert.ToInt32(OddNumbers)) * 2

        'turn it back into a string
        OddNumbers = iOddNumbers.ToString()

        ' Add the digits of the product together 
        LenOfData = OddNumbers.Length
        iOddNumbers = 0
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            iOddNumbers = iOddNumbers + Convert.ToInt32(OddNumbers.Substring(idx, 1))
            idx += 1
        Loop

        WeightedTotal = iOddNumbers + EvenNumbers

        ' Find the CheckDigit by finding the number + weightedTotal that = a multiple of 10 
        ' divide by 10, get the remainder and subtract from 10 
        iOddNumbers = WeightedTotal Mod 10
        If iOddNumbers <> 0 Then
            CheckDigit = (10 - iOddNumbers)
        Else
            CheckDigit = 0
        End If


        ' Get OutData String 
        OutData = "(" & OutData & GoodData & (CheckDigit.ToString()) & ")"
        ' Return OutData 
        Return OutData
    End Function 'End MSI


    'This will create the encoding necessary to generate a UPCA bar code.
    'The UPCA code is always 11 digits + a check character + an optional 2 or 5
    'digit add on number combination.
    Public Function UPCa(ByVal InputValue As String) As String
        Dim idx As Integer = 0 'for loop counter
        Dim LenOfData As Integer 'length of a string
        Dim GoodData As String = "" 'Invalid characters stripped from InputValue
        Dim cData As Char() 'character array representation of a string to check ascii values of characters
        Dim EAN5AddOn As String = "" '5 digit EAN Add On code
        Dim EAN2AddOn As String = "" '2 digit EAN Add On code
        Dim WeightFactor As Integer = 0 'Weighting multiplier for calculating check digit
        Dim WeightedTotal As Integer = 0 'Weighted Total for creating check digit
        Dim CheckDigit As Integer = 0 'This is the check digit for our upc code
        Dim iCurrentChar As Integer = 0 'ASCII value of current character.
        Dim iCharNumber As Integer = 0 'actual value of a numeric character string
        Dim DataToPrint As String = "" 'This is the actual encoded data once we have formatted the Input Value. This will be the function return value
        Dim Encoding As String = "" 'This is the encoding method that we are going to use for the EAN Add ons
        Dim sTemp As String = "" 'Temporary string for data manipulation
        Dim CurrEncoding As String = "" 'Encoding method for particular character
        Dim EANAddOnToPrint As String = "" 'This is the encoded EAN add on that we will add to our DataToPrint output string
        Dim EANToPrintAscii As Integer = 0 'This is the ascii character that is the encoded representation of a character in EAN add on

        cData = InputValue.ToCharArray()
        LenOfData = InputValue.Length
        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Add all numbers to GoodData string 
            If Char.IsNumber(cData(idx)) = True Then
                GoodData = GoodData & cData(idx)
            End If
            idx += 1
        Loop 'end loop for checking characters

        ' Remove check digits if they added one and retrieve the add on digits, if any. 
        'In order for this to be a valid code the length of the data can be only one of
        '3 values:  12, 14, 17.  This represents an 11 digit UPCA code, a check digit, 
        'and the options 2 or 5 digit add on code.  We strip off all check digits and 
        'recalculate it later, just to make sure it is accurate.
        Select Case GoodData.Length
            Case 12 '11 digit upc code + check digit
                GoodData = GoodData.Substring(0, 11)
            Case 14 '11 digit upc code + check digit + 2 digit add on
                EAN2AddOn = GoodData.Substring(12)
                GoodData = GoodData.Substring(0, 11)
            Case 17 '11 digit upc code + check digit + 5 digit add on
                EAN5AddOn = GoodData.Substring(12)
                GoodData = GoodData.Substring(0, 11)
                Exit Select
        End Select 'end switch for getting UPC code and Add On data

        ' <<<< Calculate Check Digit and add it to the end of the GoodData string for
        '* bar code generation>>>>
        WeightFactor = 3
        WeightedTotal = 0
        LenOfData = GoodData.Length

        For idx = (LenOfData - 1) To 0 Step -1
            ' Get the value of each character string starting at the end 
            iCharNumber = Convert.ToInt32(GoodData.Substring(idx, 1))

            ' multiply by the weighting factor which is 3,1,3,1... 
            ' and add the sum together 
            WeightedTotal = WeightedTotal + (iCharNumber * WeightFactor)

            ' change factor for next calculation 
            WeightFactor = 4 - WeightFactor
        Next idx 'end loop creating Weighted Total for Check digit

        ' Find the CheckDigit by finding the number + weightedTotal that = a multiple of 10 
        ' divide by 10, get the remainder and subtract from 10 
        CheckDigit = WeightedTotal Mod 10
        If CheckDigit <> 0 Then
            CheckDigit = 10 - CheckDigit
        Else
            CheckDigit = 0
        End If

        'Add the check digit to our GoodData string representing our actual bar code data
        GoodData = GoodData & CheckDigit.ToString()

        ' Now that have the total number including the check digit, determine the actual characters
        '* to print for proper barcoding
        LenOfData = GoodData.Length
        cData = GoodData.ToCharArray()
        DataToPrint = "" 'initialize our return value

        idx = 0
        'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < LenOfData
            ' Get the ASCII value of each number 
            iCurrentChar = Microsoft.VisualBasic.Asc(cData(idx))
            ' Get the actual value of each string value 
            iCharNumber = Convert.ToInt32(GoodData.Substring(idx, 1))

            ' Print different barcodes according to the location of the CurrentChar (idx value) 
            Select Case idx
                Case 0
                    ' For the first character print the human readable character, the normal 
                    ' guard pattern and then the barcode without the human readable character 
                    ' First get the actual value of the character to determine what to add to 
                    ' the ASCII value 
                    If iCharNumber > 4 Then
                        DataToPrint = (Microsoft.VisualBasic.Chr(iCurrentChar + 64)) & "(" & (Microsoft.VisualBasic.Chr(iCurrentChar + 49))
                    ElseIf iCharNumber < 5 Then
                        DataToPrint = (Microsoft.VisualBasic.Chr(iCurrentChar + 37)) & "(" & (Microsoft.VisualBasic.Chr(iCurrentChar + 49))
                    End If
                Case 5
                    DataToPrint = DataToPrint + cData(idx) & "*"
                Case 11
                    ' For the last character print the barcode without the human readable character, 
                    ' the normal guard pattern and then the human readable character. 
                    DataToPrint = DataToPrint + (Microsoft.VisualBasic.Chr(iCurrentChar + 59)) & "("
                    If iCharNumber > 4 Then
                        DataToPrint = DataToPrint + (Microsoft.VisualBasic.Chr(iCurrentChar + 64))
                    ElseIf iCharNumber < 5 Then
                        DataToPrint = DataToPrint + (Microsoft.VisualBasic.Chr(iCurrentChar + 37))
                    End If
                Case Else 'for 2,3,4,5
                    If idx > 0 AndAlso idx < 5 Then
                        DataToPrint = DataToPrint + cData(idx)
                    ElseIf idx > 5 AndAlso idx < 11 Then  ' this is required when printing to the right of the center guard pattern 
                        DataToPrint = DataToPrint + (Microsoft.VisualBasic.Chr(iCurrentChar + 27))
                    End If
                    Exit Select
            End Select 'End switch statement for encoding characters into output string
            idx += 1
        Loop 'End for loop going through characters to determine encoding.

        'Process the 5 digit add on, if necessary
        If EAN5AddOn.Length = 5 Then
            ' Get check digit for add on 
            WeightFactor = 3
            WeightedTotal = 0
            LenOfData = EAN5AddOn.Length

            For idx = (LenOfData - 1) To 0 Step -1
                ' Get the value of each character string starting at the end 
                iCharNumber = Convert.ToInt32(EAN5AddOn.Substring(idx, 1))

                ' multiply by the weighting factor which is 3,1,3,1... 
                ' and add the sum together 
                If WeightFactor = 3 Then
                    WeightedTotal = WeightedTotal + (iCharNumber * 3)
                ElseIf WeightFactor = 1 Then
                    WeightedTotal = WeightedTotal + (iCharNumber * 9)
                End If
                ' change factor for next calculation 
                WeightFactor = 4 - WeightFactor
            Next idx 'end loop creating Weighted Total for Check digit


            ' Find the CheckDigit by extracting the right-most number ( the value of the
            '* last character from weightedTotal
            sTemp = WeightedTotal.ToString()
            CheckDigit = Convert.ToInt32(sTemp.Substring(sTemp.Length - 1)) '-1 because substring is 0-based

            ' Now we must encode the add-on CheckDigit into the number sets 
            ' by using variable parity between character sets A and B 
            Select Case CheckDigit
                Case 0
                    Encoding = "BBAAA"
                Case 1
                    Encoding = "BABAA"
                Case 2
                    Encoding = "BAABA"
                Case 3
                    Encoding = "BAAAB"
                Case 4
                    Encoding = "ABBAA"
                Case 5
                    Encoding = "AABBA"
                Case 6
                    Encoding = "AAABB"
                Case 7
                    Encoding = "ABABA"
                Case 8
                    Encoding = "ABAAB"
                Case 9
                    Encoding = "AABAB"
                    Exit Select
            End Select 'end switch to dtermine encoding

            ' Now that we have the total number including the check digit, determine character to print 
            ' for proper barcoding: 
            LenOfData = EAN5AddOn.Length
            For idx = 0 To LenOfData - 1
                iCharNumber = Convert.ToInt32(EAN5AddOn.Substring(idx, 1)) ' Get the actual value of each number. Note that it is encoded with variable parity 
                CurrEncoding = Encoding.Substring(idx, 1)
                If CurrEncoding = "A" Then
                    Select Case iCharNumber
                        Case 0
                            EANToPrintAscii = 34
                        Case 1
                            EANToPrintAscii = 35
                        Case 2
                            EANToPrintAscii = 36
                        Case 3
                            EANToPrintAscii = 37
                        Case 4
                            EANToPrintAscii = 38
                        Case 5
                            EANToPrintAscii = 44
                        Case 6
                            EANToPrintAscii = 46
                        Case 7
                            EANToPrintAscii = 47
                        Case 8
                            EANToPrintAscii = 58
                        Case 9
                            EANToPrintAscii = 59
                            Exit Select
                    End Select 'end switch charNumber for encoding A
                ElseIf CurrEncoding = "B" Then
                    Select Case iCharNumber
                        Case 0
                            EANToPrintAscii = 122
                        Case 1
                            EANToPrintAscii = 61
                        Case 2
                            EANToPrintAscii = 63
                        Case 3
                            EANToPrintAscii = 64
                        Case 4
                            EANToPrintAscii = 91
                        Case 5
                            EANToPrintAscii = 92
                        Case 6
                            EANToPrintAscii = 93
                        Case 7
                            EANToPrintAscii = 95
                        Case 8
                            EANToPrintAscii = 123
                        Case 9
                            EANToPrintAscii = 125
                            Exit Select
                    End Select 'end switch charNumber for encoding A
                End If 'end else if CurrEncoding == B
                EANAddOnToPrint = EANAddOnToPrint + (Microsoft.VisualBasic.Chr(EANToPrintAscii))
                Select Case idx ' add in the space & add-on guard pattern 
                    Case 0 ' and Now print add-on delineators between each add-on character 
                        iCharNumber = 43
                        EANAddOnToPrint = (Microsoft.VisualBasic.Chr(iCharNumber)) + EANAddOnToPrint
                        iCharNumber = 33
                        EANAddOnToPrint = EANAddOnToPrint + (Microsoft.VisualBasic.Chr(iCharNumber))
                    Case 1
                        iCharNumber = 33
                        EANAddOnToPrint = EANAddOnToPrint + (Microsoft.VisualBasic.Chr(iCharNumber))
                    Case 2
                        iCharNumber = 33
                        EANAddOnToPrint = EANAddOnToPrint + (Microsoft.VisualBasic.Chr(iCharNumber))
                    Case 3
                        iCharNumber = 33
                        EANAddOnToPrint = EANAddOnToPrint + (Microsoft.VisualBasic.Chr(iCharNumber))
                    Case Else
                        Exit Select
                End Select 'End switch for creating EANAddOnToPrint
            Next 'end loop thru 5 digit add on
        End If 'end if 5 digit addon

        If EAN2AddOn.Length = 2 Then
            iCharNumber = Convert.ToInt32(EAN2AddOn)
            For idx = 0 To 99 Step 4
                If iCharNumber = idx Then
                    Encoding = "AA"
                ElseIf iCharNumber = idx + 1 Then
                    Encoding = "AB"
                ElseIf iCharNumber = idx + 2 Then
                    Encoding = "BA"
                ElseIf iCharNumber = idx + 3 Then
                    Encoding = "BB"
                End If
            Next idx ' end for loop determinging encoding type

            LenOfData = EAN2AddOn.Length ' Now that we have the total number including the encoding determine what to print 
            idx = 0
            'ORIGINAL LINE: for(idx = 0;idx < LenOfData;idx += 1)
            'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
            Do While idx < LenOfData
                ' Get the actual value of each number. note that it is encoded with variable parity 
                iCharNumber = Convert.ToInt32(EAN2AddOn.Substring(idx, 1))
                CurrEncoding = Encoding.Substring(idx, 1)
                If CurrEncoding = "A" Then
                    Select Case iCharNumber
                        Case 0
                            EANToPrintAscii = 34
                        Case 1
                            EANToPrintAscii = 35
                        Case 2
                            EANToPrintAscii = 36
                        Case 3
                            EANToPrintAscii = 37
                        Case 4
                            EANToPrintAscii = 38
                        Case 5
                            EANToPrintAscii = 44
                        Case 6
                            EANToPrintAscii = 46
                        Case 7
                            EANToPrintAscii = 47
                        Case 8
                            EANToPrintAscii = 58
                        Case 9
                            EANToPrintAscii = 59
                            Exit Select
                    End Select 'end switch charNumber for encoding A
                ElseIf CurrEncoding = "B" Then
                    Select Case iCharNumber
                        Case 0
                            EANToPrintAscii = 122
                        Case 1
                            EANToPrintAscii = 61
                        Case 2
                            EANToPrintAscii = 63
                        Case 3
                            EANToPrintAscii = 64
                        Case 4
                            EANToPrintAscii = 91
                        Case 5
                            EANToPrintAscii = 92
                        Case 6
                            EANToPrintAscii = 93
                        Case 7
                            EANToPrintAscii = 95
                        Case 8
                            EANToPrintAscii = 123
                        Case 9
                            EANToPrintAscii = 125
                            Exit Select
                    End Select 'end switch charNumber for encoding A
                End If 'end else if CurrEncoding == B
                EANAddOnToPrint = EANAddOnToPrint + (Microsoft.VisualBasic.Chr(EANToPrintAscii))

                If idx = 0 Then ' add in the space & add-on guard pattern 
                    iCharNumber = 43
                    EANAddOnToPrint = (Microsoft.VisualBasic.Chr(iCharNumber)) + EANAddOnToPrint
                    iCharNumber = 33
                    EANAddOnToPrint = EANAddOnToPrint + (Microsoft.VisualBasic.Chr(iCharNumber))
                End If 'End if
                idx = idx + 1
            Loop 'end loop thru 2 digit add on
        End If 'end if ean2 add on

        DataToPrint = DataToPrint + EANAddOnToPrint
        Return DataToPrint
    End Function  'end upca()

    Public Function UPCe(ByVal InputValue As String) As String
        Dim sbOutput As System.Text.StringBuilder = New System.Text.StringBuilder()
        Dim EANAddOnToPrint As System.Text.StringBuilder = New System.Text.StringBuilder()
        Dim ActualInputValue As String = "" 'This is the input value after it has been cut & cleaned
        Dim EAN2AddOn As String = "" 'optional 2 digit add on
        Dim EAN5AddOn As String = "" 'optional 5 digit add on
        Dim StringLength As Integer = InputValue.Length

        Dim idx As Integer = 0
        'ORIGINAL LINE: for(int idx = 0;idx < InputValue.Length;idx += 1)
        'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
        Do While idx < InputValue.Length
            'we are only going to allow, at most, 18 characters to be procesed
            If ActualInputValue.Length = 18 Then
                Exit Do
            ElseIf Char.IsDigit(InputValue, idx) = True Then  'Make sure we only have numbers
                ActualInputValue &= InputValue.Substring(idx, 1)
            End If
            idx += 1
        Loop

        'If the user passed in bad data or invalid length, we will return a standard value
        If ActualInputValue.Length < 11 OrElse ActualInputValue.Length = 15 OrElse ActualInputValue.Length > 18 Then
            ActualInputValue = "00005000000"
        End If

        'remove check digits so we can recalculate and pull off supplements
        If ActualInputValue.Length = 12 Then
            ActualInputValue = ActualInputValue.Substring(0, 11)
        ElseIf ActualInputValue.Length = 13 Then
            ActualInputValue = ActualInputValue.Substring(0, 11)
            EAN2AddOn = ActualInputValue.Substring(12, 2)
        ElseIf ActualInputValue.Length = 14 Then
            ActualInputValue = ActualInputValue.Substring(0, 11)
            EAN2AddOn = ActualInputValue.Substring(13, 2)
        ElseIf ActualInputValue.Length = 16 Then
            ActualInputValue = ActualInputValue.Substring(0, 11)
            EAN5AddOn = ActualInputValue.Substring(11, 5)
        ElseIf ActualInputValue.Length = 17 Then
            ActualInputValue = ActualInputValue.Substring(0, 11)
            EAN5AddOn = ActualInputValue.Substring(13, 5)
        End If

        'Calculate the check digit
        Dim Factor As Integer = 3
        Dim WeightedTotal As Integer = 0

        'we need to go backwards thru OnlyCorrectData for the check digit
        Dim I As Integer = 0
        For I = ActualInputValue.Length - 1 To 0 Step -1
            'Get the value of each number starting at the end
            Dim CurrentNumber As Integer = Convert.ToInt32(ActualInputValue.Substring(I, 1))
            WeightedTotal = WeightedTotal + (CurrentNumber * Factor)
            Factor = 4 - Factor 'change factor for next calculation
        Next I

        'Find the CheckDigit by finding the number + WeightedTotal that = a multiple of 10
        'divide by 10, get the remainder and subtract from 10
        Dim remainder As Integer = (WeightedTotal Mod 10)
        Dim CheckDigit As Integer = 0
        If remainder <> 0 Then
            remainder = 10 - remainder
            ActualInputValue &= remainder.ToString()
        Else
            ActualInputValue &= "0"
        End If

        'Compress UPC-A to UPC-E if possible
        Dim D1 As Integer = Convert.ToInt32(ActualInputValue.Substring(0, 1))
        Dim D2 As Integer = Convert.ToInt32(ActualInputValue.Substring(1, 1))
        Dim D3 As Integer = Convert.ToInt32(ActualInputValue.Substring(2, 1))
        Dim D4 As Integer = Convert.ToInt32(ActualInputValue.Substring(3, 1))
        Dim D5 As Integer = Convert.ToInt32(ActualInputValue.Substring(4, 1))
        Dim D6 As Integer = Convert.ToInt32(ActualInputValue.Substring(5, 1))
        Dim D7 As Integer = Convert.ToInt32(ActualInputValue.Substring(6, 1))
        Dim D8 As Integer = Convert.ToInt32(ActualInputValue.Substring(7, 1))
        Dim D9 As Integer = Convert.ToInt32(ActualInputValue.Substring(8, 1))
        Dim D10 As Integer = Convert.ToInt32(ActualInputValue.Substring(9, 1))
        Dim D11 As Integer = Convert.ToInt32(ActualInputValue.Substring(10, 1))
        Dim D12 As Integer = Convert.ToInt32(ActualInputValue.Substring(11, 1))

        'Condition A
        If (D11 = 5 OrElse D11 = 6 OrElse D11 = 7 OrElse D11 = 8 OrElse D11 = 9) AndAlso (D6 <> 0 AndAlso D7 = 0 AndAlso D8 = 0 AndAlso D9 = 0 AndAlso D10 = 0) Then
            ActualInputValue = String.Format("{0}{1}{2}{3}{4}{5}", D2, D3, D4, D5, D6, D11)
        ElseIf (D6 = 0 AndAlso D7 = 0 AndAlso D8 = 0 AndAlso D9 = 0 AndAlso D10 = 0) AndAlso D5 <> 0 Then  'Condition B
            ActualInputValue = String.Format("{0}{1}{2}{3}{4}{5}", D2, D3, D4, D5, D11, "4")
            'Condition C
        ElseIf (D5 = 0 AndAlso D6 = 0 AndAlso D7 = 0 AndAlso D8 = 0) AndAlso (D4 = 1 OrElse D4 = 2 OrElse D4 = 0) Then
            ActualInputValue = String.Format("{0}{1}{2}{3}{4}{5}", D2, D3, D9, D10, D11, D4)
            'Condition D
        ElseIf (D5 = 0 AndAlso D6 = 0 AndAlso D7 = 0 AndAlso D8 = 0 AndAlso D9 = 0) AndAlso (D4 >= 3 AndAlso D4 <= 9) Then
            ActualInputValue = String.Format("{0}{1}{2}{3}{4}{5}", D2, D3, D4, D10, D11, "3")
        Else
            ActualInputValue = ActualInputValue
        End If

        'Run UPC-E compression only if DataToEncode = 6
        If ActualInputValue.Length = 6 Then
            'Now we must encode the check character into the symbol
            'by using variable parity between character sets A and B
            Dim Encoding As String = ""
            Select Case D12
                Case 0
                    Encoding = "BBBAAA"
                Case 1
                    Encoding = "BBABAA"
                Case 2
                    Encoding = "BBAABA"
                Case 3
                    Encoding = "BBAAAB"
                Case 4
                    Encoding = "BABBAA"
                Case 5
                    Encoding = "BAABBA"
                Case 6
                    Encoding = "BAAABB"
                Case 7
                    Encoding = "BABABA"
                Case 8
                    Encoding = "BABAAB"
                Case 9
                    Encoding = "BAABAB"
            End Select
            I = 1
            'ORIGINAL LINE: for(int I = 1;I <= ActualInputValue.Length;I += 1)
            'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
            Do While I <= ActualInputValue.Length
                If I = 1 Then
                    'For the LeadingDigit print the human readable character, the normal guard pattern and then the rest of the barcode
                    'This is the first character so we restart the buffer from the begining
                    sbOutput.Append(Chr(85))
                    sbOutput.Append("(")
                End If
                'Get the ASCII value of each number
                Dim CurrentNumber As Integer = Microsoft.VisualBasic.Asc(ActualInputValue.Substring(I - 1, 1))

                'Print different barcodes according to the location of the CurrentChar and CurrentEncoding
                Select Case Encoding.ToCharArray()(I - 1)
                    Case "A"c
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber))
                    Case "B"c
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 17))
                End Select

                'add in the 1st character along with guard patterns
                Select Case I
                    Case 1
                    Case 6 'Print the SPECIAL guard pattern and check character
                        sbOutput.Append(")")
                        If D12 > 4 Then
                            sbOutput.Append(Microsoft.VisualBasic.Chr(Microsoft.VisualBasic.Asc(D12.ToString()) + 64))
                        End If
                        If D12 < 5 Then
                            sbOutput.Append(Microsoft.VisualBasic.Chr(Microsoft.VisualBasic.Asc(D12.ToString()) + 37))
                        End If
                        Exit Select
                End Select 'end switch
                I += 1
            Loop 'end for loop
        End If ' end if length is 6

        'if we weren't able to compress DataToEncode into 6 characters....determine character to print for proper upc-a barcoding
        If ActualInputValue.Length <> 6 Then
            i = 1
            'ORIGINAL LINE: for(int I = 1;I <= ActualInputValue.Length;I += 1)
            'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
            Do While I <= ActualInputValue.Length
                'Get the ASCII value of each number
                Dim CurrentNumber As Integer = Microsoft.VisualBasic.Asc(ActualInputValue.Substring(I - 1, 1))

                'Print different barcodes according to the location of the CurrentChar
                Select Case I
                    Case 1
                        'For the first character print the human readable character, the normal
                        'guard pattern and then the barcode without the human readable character
                        If Convert.ToInt32(ActualInputValue.Substring(I - 1, 1)) > 4 Then
                            sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 64))
                        Else
                            sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 37))
                        End If
                        sbOutput.Append("(")
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 49))
                    Case 2
                        sbOutput.Append(ActualInputValue.Substring(I - 1, 1))
                    Case 3
                        sbOutput.Append(ActualInputValue.Substring(I - 1, 1))
                    Case 4
                        sbOutput.Append(ActualInputValue.Substring(I - 1, 1))
                    Case 5
                        sbOutput.Append(ActualInputValue.Substring(I - 1, 1))
                    Case 6
                        'Print the center guard pattern after the 6th character
                        sbOutput.Append(ActualInputValue.Substring(I - 1, 1))
                        sbOutput.Append("*")
                    Case 7
                        'Add 27 to the ASII value of characters 6-12 to print from character set+ C
                        'this is required when printing to the right of the center guard pattern
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 27))
                    Case 8
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 27))
                    Case 9
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 27))
                    Case 10
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 27))
                    Case 11
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 27))
                    Case 12
                        'For the last character print the barcode without the human readable character,
                        'the normal guard pattern and then the human readable character.
                        sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 59))
                        sbOutput.Append(")")
                        If Convert.ToInt32(ActualInputValue.Substring(I - 1, 1)) > 4 Then
                            sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 64))
                        Else
                            sbOutput.Append(Microsoft.VisualBasic.Chr(CurrentNumber + 37))
                        End If
                        Exit Select
                End Select 'end switch
                I += 1
            Loop 'end for loop thru characters
        End If 'end if not 6 characters

        If EAN5AddOn.Length = 5 Then
            'Get check digit for add on
            Factor = 3
            WeightedTotal = 0

            For I = EAN5AddOn.Length - 1 To 0 Step -1
                'Get the value of each number starting at the end
                Dim CurrentNumber As Integer = Convert.ToInt32(EAN5AddOn.Substring(I, 1))
                'multiply by the weighting factor which is 3,9,3,9...and add the sum together
                If Factor = 3 Then
                    WeightedTotal = WeightedTotal + (CurrentNumber * 3)
                End If
                If Factor = 1 Then
                    WeightedTotal = WeightedTotal + (CurrentNumber * 9)
                End If

                'change factor for next calculation
                Factor = 4 - Factor
            Next I 'end loop thru characters
            'Find the CheckDigit by extracting the right-most number from WeightedTotal
            Dim sCheck As String = WeightedTotal.ToString()
            CheckDigit = Convert.ToInt32(sCheck.Substring(sCheck.Length - 1, 1))
            Dim Encoding As String = ""
            'Now we must encode the add-on CheckDigit into the number sets
            'by using variable parity between character sets A and B
            Select Case CheckDigit
                Case 0
                    Encoding = "BBAAA"
                Case 1
                    Encoding = "BABAA"
                Case 2
                    Encoding = "BAABA"
                Case 3
                    Encoding = "BAAAB"
                Case 4
                    Encoding = "ABBAA"
                Case 5
                    Encoding = "AABBA"
                Case 6
                    Encoding = "AAABB"
                Case 7
                    Encoding = "ABABA"
                Case 8
                    Encoding = "ABAAB"
                Case 9
                    Encoding = "AABAB"
            End Select

            'Now that we have the total number including the check digit, determine character to print for proper barcoding:
            I = 1
            'ORIGINAL LINE: for(int I = 1;I <= EAN5AddOn.Length;I += 1)
            'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
            Do While I <= EAN5AddOn.Length
                'Get the value of each number.  It is encoded with variable parity
                Dim CurrentNumber As Integer = Convert.ToInt32(EAN5AddOn.Substring(I - 1, 1))
                'Print different barcodes according to the location of the CurrentChar and CurrentEncoding
                Select Case Encoding.ToCharArray()(I - 1)
                    Case "A"c
                        If CurrentNumber = 0 Then
                            EANAddOnToPrint.Append(Chr(34))
                        End If
                        If CurrentNumber = 1 Then
                            EANAddOnToPrint.Append(Chr(35))
                        End If
                        If CurrentNumber = 2 Then
                            EANAddOnToPrint.Append(Chr(36))
                        End If
                        If CurrentNumber = 3 Then
                            EANAddOnToPrint.Append(Chr(37))
                        End If
                        If CurrentNumber = 4 Then
                            EANAddOnToPrint.Append(Chr(38))
                        End If
                        If CurrentNumber = 5 Then
                            EANAddOnToPrint.Append(Chr(44))
                        End If
                        If CurrentNumber = 6 Then
                            EANAddOnToPrint.Append(Chr(46))
                        End If
                        If CurrentNumber = 7 Then
                            EANAddOnToPrint.Append(Chr(47))
                        End If
                        If CurrentNumber = 8 Then
                            EANAddOnToPrint.Append(Chr(58))
                        End If
                        If CurrentNumber = 9 Then
                            EANAddOnToPrint.Append(Chr(59))
                        End If

                    Case "B"c
                        If CurrentNumber = 0 Then
                            EANAddOnToPrint.Append(Chr(122))
                        End If
                        If CurrentNumber = 1 Then
                            EANAddOnToPrint.Append(Chr(61))
                        End If
                        If CurrentNumber = 2 Then
                            EANAddOnToPrint.Append(Chr(63))
                        End If
                        If CurrentNumber = 3 Then
                            EANAddOnToPrint.Append(Chr(64))
                        End If
                        If CurrentNumber = 4 Then
                            EANAddOnToPrint.Append(Chr(91))
                        End If
                        If CurrentNumber = 5 Then
                            EANAddOnToPrint.Append(Chr(92))
                        End If
                        If CurrentNumber = 6 Then
                            EANAddOnToPrint.Append(Chr(93))
                        End If
                        If CurrentNumber = 7 Then
                            EANAddOnToPrint.Append(Chr(95))
                        End If
                        If CurrentNumber = 8 Then
                            EANAddOnToPrint.Append(Chr(123))
                        End If
                        If CurrentNumber = 9 Then
                            EANAddOnToPrint.Append(Chr(125))
                        End If
                End Select

                'add in the space & add-on guard pattern
                Select Case I
                    Case 1
                        EANAddOnToPrint.Insert(0, Chr(43)) 'add it to the front
                        EANAddOnToPrint.Append(Chr(33))
                        'Now print add-on delineators between each add-on character
                    Case 2
                        EANAddOnToPrint.Append(Chr(33))
                    Case 3
                        EANAddOnToPrint.Append(Chr(33))
                    Case 4
                        EANAddOnToPrint.Append(Chr(33))
                    Case 5
                        'EANAddOnToPrint = EANAddOnToPrint
                End Select
                I += 1
            Loop 'end for loop
        End If 'end if 5 digit add on
        If EAN2AddOn.Length = 2 Then
            'Get encoding for add on
            Dim Encoding As String = ""
            I = 0
            For I = 0 To 99
                If Convert.ToInt32(EAN2AddOn) = I Then
                    Encoding = "AA"
                End If
                If Convert.ToInt32(EAN2AddOn) = I + 1 Then
                    Encoding = "AB"
                End If
                If Convert.ToInt32(EAN2AddOn) = I + 2 Then
                    Encoding = "BA"
                End If
                If Convert.ToInt32(EAN2AddOn) = I + 3 Then
                    Encoding = "BB"
                End If
            Next I 'end for loop

            'Now that we have the total number including the encoding determine what to print
            I = 1
            'ORIGINAL LINE: for(int I = 1;I <= EAN2AddOn.Length;I += 1)
            'INSTANT VB NOTE: This 'for' loop was translated to a VB 'Do While' loop:
            Do While I <= EAN2AddOn.Length
                Dim CurrentNumber As Integer = Convert.ToInt32(EAN5AddOn.Substring(I - 1, 1))

                'Print different barcodes according to the location of the CurrentChar and CurrentEncoding
                Select Case Encoding.ToCharArray()(I - 1)
                    Case "A"c
                        If CurrentNumber = 0 Then
                            EANAddOnToPrint.Append(Chr(34))
                        End If
                        If CurrentNumber = 1 Then
                            EANAddOnToPrint.Append(Chr(35))
                        End If
                        If CurrentNumber = 2 Then
                            EANAddOnToPrint.Append(Chr(36))
                        End If
                        If CurrentNumber = 3 Then
                            EANAddOnToPrint.Append(Chr(37))
                        End If
                        If CurrentNumber = 4 Then
                            EANAddOnToPrint.Append(Chr(38))
                        End If
                        If CurrentNumber = 5 Then
                            EANAddOnToPrint.Append(Chr(44))
                        End If
                        If CurrentNumber = 6 Then
                            EANAddOnToPrint.Append(Chr(46))
                        End If
                        If CurrentNumber = 7 Then
                            EANAddOnToPrint.Append(Chr(47))
                        End If
                        If CurrentNumber = 8 Then
                            EANAddOnToPrint.Append(Chr(58))
                        End If
                        If CurrentNumber = 9 Then
                            EANAddOnToPrint.Append(Chr(59))
                        End If
                    Case "B"c
                        If CurrentNumber = 0 Then
                            EANAddOnToPrint.Append(Chr(122))
                        End If
                        If CurrentNumber = 1 Then
                            EANAddOnToPrint.Append(Chr(61))
                        End If
                        If CurrentNumber = 2 Then
                            EANAddOnToPrint.Append(Chr(63))
                        End If
                        If CurrentNumber = 3 Then
                            EANAddOnToPrint.Append(Chr(64))
                        End If
                        If CurrentNumber = 4 Then
                            EANAddOnToPrint.Append(Chr(91))
                        End If
                        If CurrentNumber = 5 Then
                            EANAddOnToPrint.Append(Chr(92))
                        End If
                        If CurrentNumber = 6 Then
                            EANAddOnToPrint.Append(Chr(93))
                        End If
                        If CurrentNumber = 7 Then
                            EANAddOnToPrint.Append(Chr(95))
                        End If
                        If CurrentNumber = 8 Then
                            EANAddOnToPrint.Append(Chr(123))
                        End If
                        If CurrentNumber = 9 Then
                            EANAddOnToPrint.Append(Chr(125))
                        End If
                        Exit Select
                End Select 'end switch
                'add in the space & add-on guard pattern
                Select Case I
                    Case 1
                        EANAddOnToPrint.Insert(0, Chr(43)) 'add it to the front
                        EANAddOnToPrint.Append(Chr(33))
                    Case 2
                End Select
                I += 1
            Loop 'end loop thru characers
        End If 'end if 2 digit add on

        'Now we have everything together
        sbOutput.Append(EANAddOnToPrint.ToString())
        Return sbOutput.ToString()
    End Function
    'This function will print a single bar code to the printer.  It takes in the name of 
    'the font to print, the encoded string, and the size of the font.  The string should 
    'be encoded by using one of the bar code functions included in this class.  Ensure 
    'that the font is installed on the computer before attempting to call this function.
    Public Sub PrintBarCode(ByVal TextFont As String, ByVal EncodedText As String, ByVal TextFontSize As Single)
        'Update the class variables
        lclFont = New Font(TextFont, TextFontSize)
        TextToPrint = EncodedText

        'create the document object that will be printed.
        Dim PrintDoc As PrintDocument = New PrintDocument()

        'Call the print page method of the Print Document.  This assigns the event handler
        'that is used for printing operations of this bar code
        AddHandler PrintDoc.PrintPage, AddressOf PrintDocHandler

        'now print the bar code.  Control will go to the event handler for any additional 
        'instructions for the printer, such as printing additional lines and
        'the location of the bar code on the page.
        PrintDoc.Print()
        Return
    End Sub

    Public Function UCC128(ByVal InputValue As String) As String
        Dim tempInput As String
        If InputValue.Substring(0, 1).Equals("") = False Then
            tempInput = "" & InputValue
        Else
            tempInput = InputValue
        End If
        Code128(tempInput, True)
    End Function

    'This is the event handler for printing bar codes
    Private Sub PrintDocHandler(ByVal sender As Object, ByVal ev As PrintPageEventArgs)
        'Vertical postion on page of bar code
        Dim yPos As Single = 100
        'Hotizontal position on the page of the bar code
        Dim xPos As Single = 100
        'Set the left margin of the page
        Dim leftMargin As Single = ev.MarginBounds.Left
        'set the top margin of the page.
        Dim topMargin As Single = ev.MarginBounds.Top

        'set the y pos to include the height of the text
        yPos = yPos + lclFont.GetHeight(ev.Graphics)

        'set the x pos to include the left margin of the page
        xPos = xPos + leftMargin

        'send the string to the printer
        ev.Graphics.DrawString(TextToPrint, lclFont, Brushes.Black, xPos, yPos, New StringFormat())
    End Sub
End Class
